
// --------------------------------------------------------------------------------------------------
// PCLink - a utility program that connects the Ginsing to your computer
// --------------------------------------------------------------------------------------------------
//
// If you would like to experiment with your Ginsing shield from your PC using software tools that run
// on your PC, you can load this program to link the PC to the Ginsing shield. This program acts as a
// relay that reads commands from your PC and sends to to the Ginsing shield, and also sends results from
// the shield back to the PC. In order for PC based tools to work this program must be running on your
// Arduino board to facilitate the link.
//
// Prerequisites:
//
// 1. An Arduino host board with a GinSing shield connected to it
// 2. A 10 uf Electrolytic capacitor ( to bridge the RESET and 5V pins on the Arduino )
// 3. A USB cable between the Arduino and the PC ( presumably the same as you use for development )
// 4. The Arduino IDE ( from which you download this program )
//
// Before you run the PC based program, you must first download this program to your Arduino using the
// Arduino IDE. Once the program has is running on the board, you will hear an audible voice promot to
// insert the 10 uF capacitor between the RESET and 5V pins onthe Arduino. The capacitor can simply be
// placed into the Ginsing shield header on the 1st and 3rd pins next to the GinSing label on the board. 
// The negative pin as marked on the capacitor should be in the RESET (1st pin on the header). This capacitor
// is required to avoid serial connection timeouts that can occur due to a limitation on the Arduino board 
// itself. Once the capacitor is inserted you can launch the PC based tools and connect to the Ginsing 
// directly from your PC.
//
// Note:
//
// When the capacitor is installed on the board you will NOT be able to use the Arduino IDE to program 
// the board -the IDE will not be able to connect ( specifically reset ) the board. To return to programming 
// with the IDE simply remove the capacitor from the header.
//
// --------------------------------------------------------------------------------------------------
//

#include <GinSing.h>                         // main include file for GinSing software library

GinSing GS;                                  // interface class for GinSing

#define rcvPin  4                            // either pin 12 or pin 4 depending on jumper setting
#define sndPin  3                            // either pin 11 or pin 3 depending on jumper setting
#define ovfPin  2                            // either pin 10 or pin 2 depending on jumper setting

// --------------------------------------------------------------------------------------------------

void setup ()
{
  
  GS.begin ( rcvPin ,                         // start GinSing shield with the communication pins
             sndPin ,                         // as specified above. The pin settings MUST
             ovfPin );                        // match the jumper settings on the board
  
  Serial.begin( 9600 );                       // open up the serial connection to the PC
 
  speak();                                    // speak connection instructions
  
  GS.reset();                                 // reset the board to default state
 
}

// --------------------------------------------------------------------------------------------------

void loop ()
{
  GinSingSerial * chip = GS.getSerial ();     // get the serial interface to the sound chip
  

  if ( Serial.available() )                   // if PC has data to send to sound chip
  {
    int incomingByte = Serial.read();         // read the PC data
    chip->write  ( incomingByte );            // send it to the sound chip
  }
  
  else if ( chip->available() )               // if sound chip has data to send to PC
  {
    int outgoingByte = chip->read();          // read the sound chip data
    Serial.write ( outgoingByte );            // send it to the PC
  }
}

// --------------------------------------------------------------------------------------------------


void speak()
{
  GinSingVoice *v = GS.getVoice();
  
  v->begin();
  
  v->setNote ( A_2 );
  v->setBlendSpeed ( 0.2f );

   GSAllophone welcome[] = { _J, _I, _NE, _SE , _I, _NGE, _PA0 ,                 // GinSing
                             _LE,  _EE, _NE, _EK, _PA0 ,                         // link
                             _R, _E, _DE, _EE,  _PA2, _PA2, _PA2, _PA2,          // ready
                             _I, _NE, _SE, _ER, _T, _PA0,                        // insert
                             _T, _E, _NE, _PA0,                                  // 10
                             _M, _IE, _KO, _R, _OE, _F, _AE, _R, _A, _ED,        // uF
                             _KE, _A, _PO, _A, _SE, _I,  _T, _OR, _PA0,          // capacitor
                             _BE,_EE, _T, _W, _EE, _EE, _NE, _PA0,               // between
                             _R,_EE ,_SE , _E , _T , _PA2 ,                      // reset
                             _A, _NE, _OD, _PA1,                                 // and
                             _F, _IE, _V, _OE, _LO, _T, _PA2,                    // 5 volt
                             _PE, _I, _NE, _Z, _PA0,                             // pins     
                             
                             _ENDPHRASE };
                                      
    v->speak (welcome);
    
    delay ( 5000 );
   
}


