
// ---------------------------------------------------------------------------------------------------------------------------------------------
// welcome to GinSing!
// ---------------------------------------------------------------------------------------------------------------------------------------------
// 
//  The GinSing library is a Arduino software layer that provides a code interface to the GinSing Arduino shield.
//
//  The GinSing library controls the Babblebot complex waveform synthesis IC on the GinSing shield. The chip contains a wealth 
//  of complex audio processing features utilizing waveform oscillators, mixers, envelope generators, etc. These components can be
//  configured to work in a variety of ways from simple polyphonic notes, to patched synthesizer, to voice synthesis. This software
//  provides an API layer that sits on top of the low level serial command / hardware register interface to make it easier to use.
//
//  This program is your introduction on how to use the GinSing library. Most programs you write will need to include some of the functions
//  in this sample program, and it is intended to show you the minimum calls needed to make the chip do something. As you get started,
//  take a quick look at GinSing.h and GinSingDefs.h; they contiain information that will help you wrap your brain aound the interface.
//
//  The GinSing library is a c++ class that you create in your code. All methods are accessed through the the class and/or its
//  component members. To get started, this example will simply open the device and cycle through its various operating modes. For
//  convenience each mode can be previewed to give you a simple idea of functionality. The modes are discussed in more detail below.
//
//  You next step in learning the interface is to look at more detailed examples of each mode. The remaining sample programs are broken
//  out based on the mode that you'd like to know more about.
//
// ---------------------------------------------------------------------------------------------------------------------------------------------


#include <GinSing.h>    // this is the only include file you need in your code

GinSing  GS;            // this creates the class library from which all methods and data are accessed
                        // you should have only one of these in your code and reference the GinSing board through it

#define rcvPin  4       // this is the pin used for receiving    ( can be either 4 or 12 )
#define sndPin  3       // this is the pin used for transmitting ( can be either 3 or 11 ) 
#define ovfPin  2       // this is the pin used for overflow control ( can be either 2 or 10 )

int numIterations = 4;  // number of times we are going to go through the preview cycle


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setup - main program entry point
// ---------------------------------------------------------------------------------------------------------------------------------------------


void setup ()
{
 
  // The interface needs to be initialized to set up some internal variables and chip state. 
  // This is a required function call and only needs to be one once.
  
  GS.begin( rcvPin , sndPin , ovfPin );
  
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// shutdown - psuedo entry point to shut the system down
// ---------------------------------------------------------------------------------------------------------------------------------------------

void shutdown()
{
   
  // Although technically you may not need to shut down the interface, it is advised when you want to stop working with it;
  // otherwise you may find sounds will continue to play even though you're done with your code.

  GS.end();

}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// loop - cycle through the modes to preview what the board can do
// ---------------------------------------------------------------------------------------------------------------------------------------------

void loop ()
{
   
  // how many times do you want to hear this? when done, shutdown the system and go home
    
   if      ( numIterations == 0 ) { numIterations--; shutdown(); return; }
   else if ( numIterations >  0 ) { numIterations--; }
   else                           { delay ( 1000 ); return; }
   
  

// Polyphonic mode (default). 
// The polyphonic mode creates 6 identical waveform oscillators mixed to equal ampltiude. This mode
// is most useful for doing musical notes or triggering harmonic voices. Once the mode is entered you can access each of the voices
// individually to change the waveform, envelope, and other parameters at will. Seting this mode again will reset all oscillators
// to their default states. See more info in GinSingPoly.h.

 
  GinSingPoly *poly = GS.getPoly();                      // get the interface for poly mode ( the value does not change once obtained )
  poly->begin();                                         // enter poly mode; this call configures the chip for the interface and sets defaults
  poly->preview();                                       // use the built-in preview function to get an idea of what this mode does
  


// Preset mode. 
// The Sound Gin chip has 31 preset sounds that can be easily triggered to create sound effects. The presets load into
// one of 2 banks, allowing up to two preset sounds to be played at the same time. The hardware is configured so that there are 3 
// waveform oscillators that each feed each of the two mixers. The preset sounds utiize the three oscillators to demonstrate the variety
// of sounds that can be created through modulation of the oscillators. GinSingDefs.h contains names of the presets for reference. See
// more info in GinSingPreset.h.


  GinSingPreset *preset = GS.getPreset();                // get the interface for preset mode
  preset->begin();                                       // enter preset mode
  preset->preview();                                     // prevew the mode



// Voice mode. 
// The chip has the ability to synthesize human (or otherwise) speech utilizing allophones. Some voice chips
// play back a series of phonemes (voice fragments) in a sequence to create speech, but this chip can morph between phonemes to create
// a more natural tranision from phoneme to phoneme. These are called allophones. The complete list of allophones is defined in
// GinSingDefs.h. See more info in GinSingVoice.h.


  GinSingVoice *voice = GS.getVoice();                    // get the interface for voice mode
  voice->begin();                                         // enter voice mode
  voice->preview();                                       // preview the mode
  
  

// Synth mode. 
// The chip can create complex sound effects by having one or more oscillators maninipulate others using a vareity of modulation,
// envelope control, and other complex waveform techniques. The presets are examples of some of the multitude of configurations
// that can be used to create complex sounds. See more info in GinSinSynth.h.


  GinSingSynth *synth = GS.getSynth();                    // get the interface for synth mode
  synth->begin();                                         // enter synth mode
  synth->preview();                                       // preview synth mode
  
}
