
// ---------------------------------------------------------------------------------------------------------------------------------------------
// voicemode - Welcome to voice mode!
// ---------------------------------------------------------------------------------------------------------------------------------------------
//
// By configuring and manipulating the oscillators on the Babblebot IC, you can create synthetic speech. Although the internal workings
// to create speech using complex wave synthesis are complicated. the GinSing library makes it easy to use. 
//
// Ginsing speaks using allophones, which are phonemes ( speech fragments ) that are blended over time to produce smooth transitions.
// A list of predefined allophones can be found in GinSingDefs.h, and can be built into phrases that will be spoken, which may also
// include vocal inflections and realtime control over pitch, speed, etc.
//
// ---------------------------------------------------------------------------------------------------------------------------------------------

#include <GinSing.h>                                 // interface headder file
GinSing       GS;                                    // create the class library from which all methods and data are accessed

#define rcvPin  4                                    // this is the pin used for receiving    ( can be either 4 or 12 )
#define sndPin  3                                    // this is the pin used for transmitting ( can be either 3 or 11 ) 
#define ovfPin  2                                    // this is the pin used for overflow control ( can be either 2 or 10 )

GinSingVoice * voice = 0x0;                          // this will point to the voice functions once the class exists


                       
// ---------------------------------------------------------------------------------------------------------------------------------------------
//  setup - system entry point for intialization
// ---------------------------------------------------------------------------------------------------------------------------------------------

void setup ()
{
  GS.begin ( rcvPin , sndPin , ovfPin );             // start the device ( you probably already know that by now )
  
  voice = GS.getVoice();                             // get the voice mode interface; in this example we share it for all functions

  voice->begin();                                    // enter voice mode
  
  warmUpVoice();                                     // run through the the scales quickly
  sing();                                            // sing a simple song
  randomHellos();                                    // say hello at random frequencies with inflection
  speakAllPhonemes();                                // run thorugh the entire phoneme set
  chant();                                           // make up some chants

  GS.end();                                          // end the device
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// warmUpVoice - simple example to run through the musical scale
// ---------------------------------------------------------------------------------------------------------------------------------------------

#define numNotes 22  // musical notes to "sing"

GSNote notes[numNotes] = { C_2, D_2, E_2, F_2, G_2, A_2, B_2, C_3, D_3, E_3, F_3, G_3, A_3, B_3, C_4, D_4, E_4, F_4, G_4, A_4, B_4, C_5 };
                      
void warmUpVoice()
{
   // run through the scale as fast as we can
    
    GSAllophone  phrase[] = { _AA , _ENDPHRASE };                   // this is what we'll say - just a simple vowel (note we must terminate!)
    
    for ( int noteIdx = 0; noteIdx < numNotes; noteIdx++ )          // run through the notes
    {
      voice->setNote ( (GSNote) notes [ noteIdx ] );                // set a new note for this phrase
      voice->speak   ( phrase );                                    // start the voice speaking
      
      delay ( voice->getMillis( phrase ) );                         // wait for it to complete - if we don't wait we can end up overrunning
                                                                    // the chip interface ( see GinSing::isReady() for details )
    }

   GSAllophone shutup[] = { _PA1 , _ENDPHRASE };                    // tell the voice to stop
   voice->speak   ( shutup );                                       // make it stop
   
   while ( !GS.isReady() ) delay ( 1 );                             // wait for all our commands to finish
     
   delay ( 1000 );                                                  // ahh, some slience
   
}
   
// ---------------------------------------------------------------------------------------------------------------------------------------------
// sing - simple example to make the voice musical
// ---------------------------------------------------------------------------------------------------------------------------------------------

void sing()
{
 // sing a simple song
   
  #define NumNotes 10
  GSNote notes[NumNotes] = { C_4 , E_4 , D_4 , F_4 , E_4 , G_4 , E_4 , F_4 , D_4 , E_4  };
  
 
  
  GSAllophone phrase[] = { _LE , _AA , _ENDPHRASE };               // here is what we will be singing
  int phraseDur = voice->getMillis ( phrase );                     // here is how long it takes
  
  for ( int loopCnt = 0; loopCnt < 2; loopCnt++ )                  // run through the note sequence a few times
  {
    for ( int noteCnt = 0; noteCnt < NumNotes; noteCnt++ )         // run through the notes of the song
    {
        voice->setNote ( notes [ noteCnt ] );                      // set the musical note to sing
        voice->speak   ( phrase );                                 // sing the phrase
  
        delay ( 500 );                                             // wait 1/2 second before moving on
    }
  }  
  
  GSAllophone shutup[] = { _PA1 , _ENDPHRASE };                    // tell the voice to stop
  voice->speak   ( shutup );                                       // make it stop
  
  while ( !GS.isReady() ) delay ( 1 );                             // wait for all our commands to finish
    
  delay ( 1000 );                                                  // ahh, some slience
  
}





// ---------------------------------------------------------------------------------------------------------------------------------------------
// randomHellos - say hello with random pitch
// ---------------------------------------------------------------------------------------------------------------------------------------------

void randomHellos()
{
 
    GSAllophone  hello[] = { _HE , _PITCHUP , _E , _LO ,_PITCHDN ,  _OE , _PA1 , _ENDPHRASE };    // hello 
    GSAllophone  world[] = { _W , _BENDUP , _ER , _LE , _ED , _PA1 , _ENDPHRASE };                // world
    
    
    for ( int freqStep = 0; freqStep < 16; freqStep++ )                  // run through a few times
    {
      float freqHz = (float) random ( 100 , 1000 );                       // compute random hello frequency
      voice->setFrequency ( freqHz );                                    // set the frequency
      voice->speak ( hello );                                            // speak hello
      delay ( voice->getMillis( hello ) );                               // wait for it to finish
      
      freqHz = (float) random ( 100 , 1000 );                             // compute random world frequency
      voice->setFrequency ( freqHz );                                    // set the frequency
      voice->speak ( world );                                            // speak world
      delay ( voice->getMillis( world ) );                               // wait for it to finish
      
    }
    
    GSAllophone shutup[] = { _PA1 , _ENDPHRASE };                        // tell the voice to stop
    voice->speak   ( shutup );                                           // make it stop
  
    while ( !GS.isReady() ) delay ( 1 );                                 // wait for all our commands to finish
    delay ( 1000 );                                                      // ahh, some slience
    
    voice->begin();                                                      // reset the voice to a clean slate
}
    
 
// ---------------------------------------------------------------------------------------------------------------------------------------------
// speakAllPhonemes - run through the complete phoneme set
// ---------------------------------------------------------------------------------------------------------------------------------------------
 
void speakAllPhonemes()
{

  GSAllophone phoneme[] = { _ENDPHRASE, _ENDPHRASE };                      // we'll fill in the first phoneme as we go along

  for ( int phonIdx = (int) _A; phonIdx <= (int) _FD2; phonIdx++ )         // run through all the available phonemes
  {
    phoneme[0] = (GSAllophone) phonIdx;                                    // set the first phoneme in the phrase
		
    voice->speak ( phoneme );                                              // speak the phrase
    delay ( 3 * voice->getMillis ( phoneme ) );                            // give us time to hear it
  }
  
     
  GSAllophone shutup[] = { _PA1 , _ENDPHRASE };                            // tell the voice to stop
  voice->speak   ( shutup );                                               // make it stop
  
  while ( !GS.isReady() ) delay ( 1 );                                     // wait for all our commands to finish
  delay ( 1000 );                                                          // ahh, some slience
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// chant - spout some jibberish in a singing like voice
// ---------------------------------------------------------------------------------------------------------------------------------------------

void chant()
{
  
   #define NumConsonants 35
     
    GSAllophone consonants[NumConsonants ] =
   {
    _BE  , _BO , _CH , _DE , _DO  , _F  , _GE , _GO  , _HE  , _HO  ,       // these are all the available consontants
    _J   , _KE , _KO , _LE , _LO  , _M  , _NE ,  _NO , _NGE , _NGO ,       // usually used at the start and end of a word
    _PE  , _PO , _R  , _SE , _SO  , _SH , _T  , _TH  , _THH , _TS  ,
    _V   , _W  , _Y  , _Z  , _ZH 
   };
   
   #define NumVowels 23
   
   GSAllophone vowels[NumVowels] =
   {
     _A  , _AA , _AE , _AIR , _AU , _EB , _OB , _ED , _OD , _E   ,         // these are the available vowels (mostly)
     _EE , _ER , _I  , _IE  , _EK , _OK , _OE , _OI , _OO , _OU  ,         // usually used at the middle of a word
     _OR , _U  , _UE 
   }; 
   
   
   
  voice->setNote ( C_3 );                                                  // singish frequency
  
  GSAllophone jibber[] = { _ENDPHRASE, _ENDPHRASE , _ENDPHRASE };         // we'll fill in the first two phonemes as we go along

  GSAllophone inflection;                                                 // this will precede our phoneme
  GSAllophone phoneme;                                                    // this is the phoneme that will get inflected
  
  for ( int jibberCnt = 0; jibberCnt < 100; jibberCnt++ )
  {
    
    inflection = (GSAllophone) random ( _SPEEDUP , _BENDDN + 1 );           // select a random inflection
    
    if ( random ( 0 , 4 ) > 0 )                                            // most of the time
      phoneme = vowels [ random ( 0 , NumVowels ) ];                       // selet a random vowel
    else                                                                   // rarely select a consonant
      phoneme = consonants [ random ( 0 , NumConsonants ) ];               // select a random consontant
    
    jibber[0] = inflection;                                                // load up our selected inflection
    jibber[1] = phoneme;                                                   // load up our selected phoneme
		
    voice->speak ( jibber );                                               // speak the phrase
    delay ( 3 * voice->getMillis ( jibber ) );                             // give us time to hear it
  }
  
     
  GSAllophone shutup[] = { _PA1 , _ENDPHRASE };                            // tell the voice to stop
  voice->speak   ( shutup );                                               // make it stop
  
  while ( !GS.isReady() ) delay ( 1 );                                     // wait for all our commands to finish
  delay ( 1000 );                                                          // ahh, some slience
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// shutdown - function to terminate the system
// ---------------------------------------------------------------------------------------------------------------------------------------------


void shutdown()
{
  
  // goodbye
  
  GSAllophone phrase[] = {  _GO ,  _OO , _OO , _DE , _PA2 , _BE , _IE , _ENDPHRASE };
  voice->setNote ( F_3 );
  voice->speak   ( phrase );
  delay ( voice->getMillis ( phrase ) );


}



void loop (){}

