
// ---------------------------------------------------------------------------------------------------------------------------------------------
// Welcome to synth mode!
// ---------------------------------------------------------------------------------------------------------------------------------------------
// 
// All of the operations of the Sound Gin chip are rooted in waveform synthesis. Waveform synthesis involves the mixing and modulating of various
// basic waveform shapes to create complex sounds. This code covers the basic sound generation functions; for advanced features see the additional
// examples.
//
// In synth mode, six oscillators are configured into two functional banks that can be mixed independently ( bank A, bank B ).  Each bank contains 
// 3 oscillators each ( 1,2,3 ), and each has its own specific functionality as a basic tone generator, amplitude modulator, frequency modulator,
// and in some cases a pulse width modulator. There are additional features that allow you to combine the banks for special purposes which will
// be covered in other examples.
//
//
//                    BANK    (A/B)          
//      ------------------------------------------
//
//                         OSC 1 (TONE)  -> |  M           Although all oscillators can be sent to the main mix, oscillator 2 is primarily
//                             ^    ^       |  I           used to amplitude modulate oscillator 1, and oscillator 3 is primarily used to
//                             |    |       |  X           frequency modulate oscillator 1. This standard configuration can be represented as:
//       OSC 2 (AMPMOD) -------+ -- |-----> |  E
//       OSC 3 (FRQMOD) ------------+-----> |  R               [OSC] -> [MIX]      [OSC2] -> AMPMOD [OSC1]       [OSC3] -> FRQMOD [OSC1]
//
//
//
// This code example illustrates the basic synth mode interface and introduces basic synthesizer concepts - these don't make great sounds but
// do illustrate the point. Note in most instances the code does not directly maniuplate any of the oscilloator parameters in realtime, which
// can add another dimension to the sounds you create.
//
//    1.  Waveform Selection     - cycle through the waveform types
//    2.  Frequency Beating      - two waveforms with slightly different frequencies produce a beating pattern
//    3.  Frequency Distortion   - built-in function that adds distortion to an oscillator
//    4.  Envelope Control       - what an ADSR is and what it does
//    5.  Amplitude Modulation   - modulate the amplitude of one oscillator by another
//    6.  Frequency Modulation   - modulate the frequency of one oscillator by another
//    7.  Pulse Width Modulation - pulse width modulate one oscillator with another
//    8.  Frequency Ramp         - use internal oscillator function to perform frequency sweeps
//    9.  Frequency Target       - use internal oscillator fucntion to blend between target frequencies
//   10.  Amplitude Target       - use internal oscillator fucntion to blend between target amplitudes
//
// ---------------------------------------------------------------------------------------------------------------------------------------------


#include <GinSing.h>                                 // interface header file

GinSing  GS;                                         // create the class library from which all methods and data are accessed
GinSingSynth *s;                                     // our global pointer to the synth mode interface ( set up after initialization )

#define rcvPin  4                                    // this is the pin used for receiving    ( can be either 4 or 12 )
#define sndPin  3                                    // this is the pin used for transmitting ( can be either 3 or 11 ) 
#define ovfPin  2                                    // this is the pin used for overflow control ( can be either 2 or 10 )

// ---------------------------------------------------------------------------------------------------------------------------------------------

void setup ()
{
  GS.begin ( rcvPin , sndPin , ovfPin );             // start the device (required)
  
  s = GS.getSynth();                                 // get the synth mode interface
  s->begin ();                                       // enter synth mode


  cycleWaveforms();                                  // run through the waveform types
  beatFrequency();                                   // simple example of two waves beating on near frequency
  distortFrequency();                                // built-in function that adds distortion to an oscillator
  envelopeControl();                                 // what an ADSR is and what it does
  amplitudeModulation();                             // amplitude modulate one oscillator with another
  frequencyModulation();                             // frequency modulate one oscillator with another
  pwModulation();                                    // pulse width modulate one oscillator with another
  frequencyRamp();                                   // use internal oscillator function to perform frequency sweeps
  frequencyTarget();                                 // use internal oscillator fucntion to blend between target frequencies
  amplitudeTarget();                                 // use internal oscillator fucntion to blend between target amplitudes
 
  GS.end();                                          // end the device

}

// ---------------------------------------------------------------------------------------------------------------------------------------------
//  cycleWaveforms - run through the waveform types
// ---------------------------------------------------------------------------------------------------------------------------------------------

void cycleWaveforms()
{
  // This example has two purposes. First, it gives you an idea of how the interface is used to select banks, configure oscillators, 
  // waveforms; you'll be doing these a lot. Second, it lets you hear the available waveforms. Note is that the sawtooth wave and
  // the ramp wave don't have any actualy tonal difference, but they make a big difference when you use them to modulate something else. 
  // The level waveform is not audible, but it used stricly for modulation purposes.
  
  
  s->selectBank   ( BANK_A );					        // select bank A. This call simplifies the interface by allowing you to
                                                                        // specify a target for the oscillator functions rather than specifying it
                                                                        // every time. All oscillator subsequent calls will refer to this bank.
  
  s->setPatch     ( OSC_1_TO_MIXER );                                   // path oscillator 1 to the mixer. A patch is the method by which
                                                                        // you route the oscillators to each other and to the mixer. There
                                                                        // are many patch options, and they can all be bitwise or'ed to set up
                                                                        // complex arrangements.

  s->setFrequency ( OSC_1 , 1000.0f );                                  // set the oscillator to 100 Hz
  s->setAmplitude ( OSC_1 , 1.0f );                                     // set amplitude to full volume
  

  s->setWaveform  ( OSC_1 , SINE );      delay ( 2000 );                // select sinusoidal ( single frequency )
  s->setWaveform  ( OSC_1 , TRIANGLE );  delay ( 2000 );                // select trianlge ( linear rise, linear fall )
  s->setWaveform  ( OSC_1 , SAWTOOTH );  delay ( 2000 );                // select sawtooth ( instant rise, linear fall )  
  s->setWaveform  ( OSC_1 , RAMP );      delay ( 2000 );                // select ramp ( linear rise, instant fall )                                                    
  s->setWaveform  ( OSC_1 , PULSE );     delay ( 2000 );                // selet pulse ( instant rise, fall 50% duty cycle by default )                                                     
  s->setWaveform  ( OSC_1 , NOISE );     delay ( 2000 );                // select random amplitude steps                                                      
  s->setWaveform  ( OSC_1 , LEVEL );     delay ( 2000 );                // select fixed DC offset ( only useful for modulation )                                                   
 
  s->setAmplitude ( OSC_1 , 0.0f );                                     // set amplitude to zero (off)
  delay ( 1000 );	                                                // one second of silence				
}
  

// ---------------------------------------------------------------------------------------------------------------------------------------------
//  breatFrequency - simple example to triger two oscillators to observe frequency beating
// ---------------------------------------------------------------------------------------------------------------------------------------------

void beatFrequency()
{
   // Frequency beating is caused when you have two waveforms at nearly the same frequency. As the waves are added (mixed), their amplitudes
   // cycle between doubling in volume and cancelling each other out at rate that is determined by the difference in frequencies. In this 
   // example the two waveforms beat at 1 Hz. 
  
  
  s->selectBank   ( BANK_A );					        // select bank A. This call simplifies the interface by allowing you to
                                                                        // specify a target for the oscillator functions rather than specifying it
                                                                        // every time. All oscillator subsequent calls will refer to this bank.
  
  s->setPatch     ( OSC_1_TO_MIXER | OSC_2_TO_MIXER );                  // patch oscillators 1 and 2 to the mixer. A patch is the method by which
                                                                        // you connect the oscillators together and route them to the mixer. There
                                                                        // are many patch options, and they can all be bitwise or'ed to set up
                                                                        // complex arrangements
                                                                        
  s->setWaveform  ( OSC_ALL , SINE );                                   // set all oscillastors (in current bank) to use a sinusoidal waveform.

  s->setAmplitude ( OSC_1 , 0.5f );  s->setAmplitude ( OSC_2 , 0.5f );  // set fixed output amplitudes to 50% each
                                                                        
  s->setFrequency ( OSC_1 , 100.0f );                                   // set the first oscillator to 100 Hz
  s->setFrequency ( OSC_2 , 101.0f );                                   // set the second oscillator to 100 Hz. This creates a beat frequency of
                                                                        // of 1 Hz as the two waveforms drop in and out of phase, creating either
                                                                        // a doubling or zeroing of the output when added together.

  delay ( 10000 );                                                      // listen for 10 seconds

  s->setAmplitude ( OSC_1 , 0.0f );  s->setAmplitude ( OSC_2 , 0.0f );  // zero output amplitudes

  delay ( 1000 );                                                       // one second of silence
  
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
//  distortFrequency - built-in function that adds distortion to an oscillator
// ---------------------------------------------------------------------------------------------------------------------------------------------

void distortFrequency()
{
  // Frequency distortion is an easy way to add frequency depth to a tone. This feature is available on each oscillator
  // and can be useful for creating interesting whistling sounds. In this example we trigger distortion at 50%
  
   s->selectBank   ( BANK_A );					        // select bank A
  
  s->setPatch     ( OSC_1_TO_MIXER  );                                  // patch oscillator 1  to the mixer                                                              
  s->setWaveform  ( OSC_1 , SINE );                                     // set oscillator 1 to a sine wave
  s->setAmplitude ( OSC_1 , 1.0f );                                     // set oscillator 1 amplitude to 100%
  s->setFrequency ( OSC_1 , 400.0f );                                   // set the first oscillator to 400 Hz
  s->setFreqDist  ( OSC_1 , 0.5f );                                     // set frequency distortion to 50%

  delay ( 10000 );                                                      // listen for 10 seconds

  s->setFreqDist  ( OSC_1 , 0.0f );                                     // disable freuency distortion
  s->setAmplitude ( OSC_1 , 0.0f );                                     // zero output amplitudes

  delay ( 1000 );                                                       // one second of silence
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
//  envelopeControl - what an ADSR is and what it does
// ---------------------------------------------------------------------------------------------------------------------------------------------

void envelopeControl()
{
  // ADSR is an acronym for [A]ttack, [D]ecay, [S]ustain, [R]elease. You can make any oscillator use ADSR whenever you use the trigger()/release()
  // methods. You can also use ADSR on each of the two bank mixers in the GinSingMaster class. ADSR is a time-based volume control, often called
  // an envelope. When you first call trigger(), the envelope will start at zero volume and proceed to an initial amplitude over a specified
  // amount of time. These two initial values are called attack time, and attack Amplitude. After the attack time has finished, the decay amplitude
  // stage begins over the specified decay time. Likewise, after the decay stage has completed, you enter the sustain stage. The sustain stage is
  // unique in that it will hold the the decay ampltidue until you call release() and therefore has no actual time or amplitude value. When you do 
  // call release(), you transition into the release stage and move towards the release value over the course of the release time. This 4 stage 
  // amplitude envelope can be used to simulate amplitude variations commonly found in real world oscilations, such as plucked strings, percussive
  // instruments, and so on. Note that when you use envelopes you can't specify the output amplitude as in the previous examples.
  //
  // One thing to keep in mind is that the times are actually rates. For example, if you start your release with an initial value of 0.5, it will only
  // take half as long to reach zero as it would if you started with 1.0
  
 
  s->selectBank   ( BANK_A );					        // select bank A
  s->setPatch     ( OSC_1_TO_MIXER  );                                  // send oscillator 1 to mixer
  s->setWaveform  ( OSC_1 , SINE    );                                  // set oscillator 1 to sine
  s->setFrequency ( OSC_1 , 1000.0f );                                  // set frequency to 100 Hz
 
 
 // set our first envelope will be a quick rise gentle fall (like a bell)
 
  s->setEnvelope  ( OSC_1 ,                                             // set envelope for this oscillator
                    /* ATTACK  */  AT_2MS    , 1.0f ,                   // set the attack stage to reach 100% volume in 2 milliseconds
                    /* DECAY   */  DR_575MS  , 0.8f ,                   // after that, drop the amplitude down to 80% in about 1/2 second
                    /* SUSTAIN */                                       // no paramters needed - just sustain the decay value until released
                    /* RELEASE */  DR_1500MS , 0.0f );                  // when release() is called, fade out to 0% volume in 1.5 seconds
  
  for ( int loopIdx = 0; loopIdx < 4; loopIdx++ )                       // lets hear this a few times
  {
    s->trigger( OSC_1 );                                                // trigger
    delay ( 1077 );                                                     // 0.577s attack & decay , 0.5s sustain
  
    s->release( OSC_1 );                                                // start release stage
    delay ( 1500 );                                                     // 1.2s relase ( 80% of 1500ms ), .3s silence
  }
  
  
  // our next envelope will uniquely synth ( reverse bell )
  
    s->setEnvelope  ( OSC_1 ,                                           // set envelope for this oscillator
                    /* ATTACK  */  AT_800MS , 1.0f ,                    // set the attack stage to reach 100% volume in 800 milliseconds
                    /* DECAY   */  DR_2MS   , 1.0f ,                    // effectively no decay
                    /* SUSTAIN */                                       // no paramters needed - just sustain the decay value until released
                    /* RELEASE */  DR_10MS  , 0.0f );                   // 10ms quick release
  
  for ( int loopIdx = 0; loopIdx < 4; loopIdx++ )                       // lets hear this a few times
  {
    s->trigger( OSC_1 );                                                // trigger
    delay ( 1002 );                                                     // 0.802ms attack, 0.200 sustain
  
    s->release( OSC_1 );                                                // start release stage
    delay ( 310 );                                                      // 0.01s release, 0.3s silence
  }
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
//  amplitudeModulation - amplitude modulate one oscillator from another ( oscillator 2 )
// ---------------------------------------------------------------------------------------------------------------------------------------------

void amplitudeModulation()
{
  // Amplitude modulation can be used the modulate the volume level of one oscillator by another. In each bank the #2 oscillators are available
  // to amplitude modulate (AM) the #1 oscillators. In order to do this, you simply patch them using the setting below. In this example, we
  // will set up a triangle wave generator as the base tone generator (1), and then set the AM modulator (2) as a pulse wave that will set
  // the output amplitude. Note that we are setting the waveform mode for the AM oscillator to positive, which means its output goes from zero
  // to the specified ampltidue; if we used the default symmetrical waveform it would only change the phase of the base oscillator.
  
  s->selectBank   ( BANK_A );					        // select bank A for target functions
  
  s->setPatch     ( OSC_1_TO_MIXER | OSC_2_AMPMOD_OSC_1 );              // patch osc 1 to mixer, and set osc 2 to amplitude modulate osc 1
  
  s->setWaveform  ( OSC_1 , TRIANGLE );			                // set the tone oscillator (1) as a triangle wave
  s->setFrequency ( OSC_1 , 1000.0f );				        // set the tone oscillator (1) to 1 kHz
  s->setAmplitude ( OSC_1 , 0.5f );                                     // set base oscillator (1) to 50% volume
  
  s->setWaveform  ( OSC_2 , PULSE );			                // set the AM oscillator (2) as a pulse wave
  s->setDutyCycle ( OSC_2 , 0.50f );                                    // set the AM oscillator (2) to 50% duty cycle ( square wave )
  
  s->setWavemode  ( OSC_2 , POSITIVE );				        // set the AM oscillator (2) to go from zero to full (no zero crossing)
                                                                        // if we used the default symmetrical waveform the output would only
                                                                        // change phase as we would send +/- instead of 0/+ with this
                                                                        
  s->setFrequency ( OSC_2 , 1.0f );					// set the AM oscillator (2) period to 1 second
  s->setAmplitude ( OSC_2 , 1.0f );                                     // set the AM oscillator (2) to full modulation range

  delay ( 10000 );                                                      // listen for 10 seconds

  s->setAmplitude ( OSC_1 , 0.0f );  s->setAmplitude ( OSC_2 , 0.0f );  // zero output amplitudes

  delay ( 1000 );                                                       // one second of silence
  
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
//  frequencyModulation - frequency modulate one oscillator with another ( oscillator 3 )
// ---------------------------------------------------------------------------------------------------------------------------------------------

void frequencyModulation()
{
  // Frequency modulation allows you to change the frequency of one oscillator through the otuput of another. In each bank the #3 oscillator
  // can be configured to frequency modulate (FM) the #1 oscillator. In this example, we create a base sinewave oscillator (1) and modulate it
  // through the FM oscillator (3).
  
  s->selectBank   ( BANK_A );					        // select bank A for target functions
  
  s->setPatch     ( OSC_1_TO_MIXER | OSC_3_FRQMOD_OSC_1 );              // patch osc 1 to mixer, and set osc 3 to frequency modulate osc 1
  
  s->setWaveform  ( OSC_1 , SINE );			                // set the tone oscillator (1) as a sine wave
  s->setFrequency ( OSC_1 , 20.0f );				        // set the tone oscillator (1) to 1 kHz
  s->setAmplitude ( OSC_1 , 0.5f );                                     // set the tone oscillator (1) volume to 50%
  
  s->setWaveform  ( OSC_3 , SINE );			                // set the FM oscillator (3) as a sine wave

  s->setWavemode  ( OSC_3 , POSITIVE );				        // set the FM oscillator (3) to go from zero to full (no zero crossing)                                                                        
  s->setFrequency ( OSC_3 , 1.0f );					// set the FM oscillator (3) period to 1 second
  s->setAmplitude ( OSC_3 , 1.0f );                                     // set the FM oscillator (3) to full frequency range of modulation
  
 
  delay ( 10000 );                                                      // listen for 10 seconds

  s->setAmplitude ( OSC_1 , 0.0f );  s->setAmplitude ( OSC_2 , 0.0f );  // zero output amplitudes

  delay ( 1000 );                                                       // one second of silence
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
//  pwModulation - pulse width modulate one oscillator with another
// ---------------------------------------------------------------------------------------------------------------------------------------------

void pwModulation()
{
  // The pulse width of the bank A oscillators can be modified by the bank B oscillators indivually. This linkage can be applied on any of
  // the oscillator sets, so its possible to mix this with standard configurations to create new effects. In these types of 'reaching acroos the isle'
  // its better to consider all oscilalators working as a single combined bank. Note that only Bank B oscillators can perform PWM, and then can only
  // modulate bank A oscillators. In this example, we configure B3 to PWM A1.

   s->begin();                                                           // start with a clean slate
  
   s->selectBank   ( BANK_A );					         // set up bank A
                   
   s->setPatch     ( OSC_B1_PWM_OSC_1  | OSC_1_TO_MIXER );               // set the PWM oscillator (B1) to link to bank A's FM oscillator (A1)
                       
   s->setWaveform  ( OSC_1 , PULSE );			                 // set the tone oscillator (1) as a pulse wave (to be modulated)
   s->setFrequency ( OSC_1 , 200.0f );				         // set the tone oscillator (1) to 200 Hz
   s->setAmplitude ( OSC_1 , 0.5f );                                     // set the tone oscillator (1) volume to 50%

    
   s->selectBank   ( BANK_B );			                         // set up bank B
   
   s->setPatch     ( OSC_B1_PWM_OSC_1 );                                 // set the PWM oscillator (B1) to link to bank A's FM oscillator (A1)
   
   s->setWaveform  ( OSC_1 , SINE );			                 // set the PWM oscillator (B1) as a sine wave
   s->setWavemode  ( OSC_1 , POSITIVE );				 // set the PWM oscillator (B1) to go from zero to full (no zero crossing)
   s->setFrequency ( OSC_1 , 0.50f );					 // set the PWM oscillator (B1) period to 2 seconds
   s->setAmplitude ( OSC_1 , 1.0f );                                     // set the PWM oscillator (B1) to full modulation range
  
   delay ( 10000 );                                                      // listen for 10 seconds

   s->selectBank ( BANK_A ); s->setAmplitude ( OSC_1 , 0.0f );           // zero output for tone oscillator
   s->selectBank ( BANK_B ); s->setAmplitude ( OSC_1 , 0.0f );           // zero output for PWM oscillator

   delay ( 1000 );                                                       // one second of silence
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
//  frequencyRamp - use internal oscillator function to perform frequency sweeps
// ---------------------------------------------------------------------------------------------------------------------------------------------

void frequencyRamp()
{
  // Each oscillator has a built in function that allows you to sweep the entire frequency range automatically without the need to 
  // use another oscillator for modulation. This is very usefuil to extend the funtionality, but does has some limits as you can't control
  // the frequency range as you can with FM. In this example we'll use tone oscillator(1) and enable frequency ramping.
  
  
   s->selectBank     ( BANK_A );				         // select bank A for target functions
   s->setPatch       ( OSC_1_TO_MIXER  );                               // patch osc 1 to mixer
  
   s->setWaveform    ( OSC_1 , SINE );			                // set the tone oscillator (1) as a sine wave
   s->setAmplitude   ( OSC_1 , 0.5f );                                  // set the tone oscillator (1) volume to 50%
   s->enableFreqRamp ( OSC_1 , true );                                  // enable frequency ramping
   s->setFreqRamp    ( OSC_1 , 0.01f );                                 // frequency ramp at a 1% relative rate

   delay ( 10000 );                                                     // listen for 10 seconds

   s->enableFreqRamp ( OSC_1 , false );                                 // disable frequency ramp
  
   delay ( 1000 );                                                      // one second of silence

}

// ---------------------------------------------------------------------------------------------------------------------------------------------
//  frequencyTarget  - use internal oscillator fucntion to blend between target frequencies
// ---------------------------------------------------------------------------------------------------------------------------------------------

void frequencyTarget()
{	
  // Frequency targeting allows you to bend between the current oscillator frequency and a specified target. The rate at which you bend
  // the frequencies is a parameter to the blending and is a relative value. In this example we loop through a few random frequencies
  // and blend towards it.

  s->selectBank   ( BANK_A );					        // select bank A
  s->setPatch     ( OSC_1_TO_MIXER  );                                  // set tone oscillator (1) to mixer
  s->setWaveform  ( OSC_1 , SINE    );                                  // set tone oscillator (1) to sine
  s->setAmplitude ( OSC_1 , 0.5f    );                                  // set tone oscillator (1) to 50% volume

  s->enableFreqTarget ( OSC_1 , true );                                 // enable frequency target
  s->enableFreqRamp   ( OSC_1 , true );                                 // enable frequency ramp (reqd for target)
  
  for ( int loopCnt = 0; loopCnt < 64; loopCnt++ )                      // loop through some target frequencies
  {
    float tgtFrq = 20.0f + (float) random ( 0 , 5000 );                 // compute a random frequency
    s->setFreqTarget ( OSC_1 , tgtFrq , 0.008f );                       // target the frequency ( relative rate 5% )
    delay ( 250 );                                                      // wait to hear it
  }

  s->enableFreqTarget( OSC_1 , false );                                 // disable frequency target
  s->enableFreqRamp  ( OSC_1 , false );                                 // disable frequency ramp
 
  delay ( 1000 );                                                       // one second of silence
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
//  ampltiudeTarget  - use internal oscillator fucntion to blend between target amplitudes
// ---------------------------------------------------------------------------------------------------------------------------------------------

void amplitudeTarget()
{
  // Amplitude targeting allows you to bend between the current oscillator amplitude and a specified target. The rate at which you bend
  // the amplitude is a parameter to the blending and is a relative value. In this example we loop through a few random amplitudes
  // and blend towards it.
  
  s->selectBank      ( BANK_A );				        // select bank A
  s->setPatch        ( OSC_1_TO_MIXER  );                               // set tone oscillator (1) to mixer
  s->setWaveform     ( OSC_1 , SINE    );                               // set tone oscillator (1) to sine
  s->setFrequency    ( OSC_1 , 200.0f    );                             // set tone oscillator (1) to 200 Hz
  s->enableAmpTarget ( OSC_1 , true );                                  // enable amplitude targeting
  s->enableAmpRamp   ( OSC_1 , true );                                  // enable amplitude ramp ( reqd for targeting )
  
  for ( int loopCnt = 0; loopCnt < 64; loopCnt++ )                      // loop through some target amplitudes
  {
    float tgtAmp = ( (float) random ( 0 , 256 ) / 255.0f );             // select a random amplitude
    s->setAmpTarget ( OSC_1 , tgtAmp , 0.01f );                         // target the amplitude ( at a relative rate )
    delay ( 250 );                                                      // wait to hear it
  }

  s->enableAmpTarget ( OSC_1 , false );                                 // disable amplitude targeting
  s->enableAmpRamp   ( OSC_1 , false );                                 // disable amplitude ramp
  
  delay ( 1000 );                                                       // one second of silience
}


void loop ()
{
}
