// ---------------------------------------------------------------------------------------------------------------------------------------------
// ginsing - a sound and voice synthesizer library to support the arduino ginsing shield
// ---------------------------------------------------------------------------------------------------------------------------------------------
// 
//  Version:  4.0
//  Creator:  www.ginsingsound.com
//
// ---------------------------------------------------------------------------------------------------------------------------------------------

#include "GinSing.h"
#include "GinSingSerial.h"


// global pointer to serial inteface instance for the device

GinSingSerial gSerialInterface;


// global pointer to the library class

GinSing * gsPtr = 0x0;


// internal function useful for seeing what the chip settings are ( diagnostic )

void _dumpRegisters();


// ---------------------------------------------------------------------------------------------------------------------------------------------
// constructor
// ---------------------------------------------------------------------------------------------------------------------------------------------

GinSing::GinSing() : serialInt ( 0x0 ) 
{
	// save off the instance pointer for all sub-classes
	
	gsPtr = this;
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// destructor
// ---------------------------------------------------------------------------------------------------------------------------------------------

GinSing::~GinSing()
{
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// begin - begin communication with the device
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSing::begin( int recvPin , int xmitPin , int ovfPin )
{
	
	// embed the serial interface class into library
	
	serialInt = & gSerialInterface;
		
	
	// configure sound board serial port communications
	
	serialInt->config ( (uint8_t) recvPin , (uint8_t) xmitPin , (uint8_t) ovfPin );
	serialInt->begin  ( 9600 );
	
	
	// reset the board to known <quiet> state
	
	reset();
	
}



// ---------------------------------------------------------------------------------------------------------------------------------------------
// end - disconnect from the device
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSing::end()
{

	// reset the system
	
	gsPtr->master.reset ();
	
	
	// terminate the serial connection
	
	serialInt->end();
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// reset - reset the device to known default state
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSing::reset()
{
	// send a reset to the board ( 0 , CmdHeader , CmdHeader )

	ubyte resetCmd[3] = { 0, CmdHeader, CmdHeader };
	
	for ( int serialIdx = 0; serialIdx < 3; serialIdx++ ) serialInt->write ( resetCmd [serialIdx] );
	
	
	// wait up to 1/2 second to reset to complete
	
	delay ( 500 );	
	
	
	// flush read buffer
		
	while (  serialInt->peek() != -1 ) serialInt->read();
	
}

		
// ---------------------------------------------------------------------------------------------------------------------------------------------
// isReady - true if ready to accept commands, false if serial line is busy
// ---------------------------------------------------------------------------------------------------------------------------------------------

bool GinSing::isReady()
{
	// return false until instanced, then return status of the send buffer
	
	if ( !gsPtr ) return ( false );
	return ( serialInt->sndOverflow()  ? false : true );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// sendCommand - send a specific low level command ( add associated data ) to the sound chip
// ---------------------------------------------------------------------------------------------------------------------------------------------
					  
void  GinSing::sendCommand ( GSCommand cmdIdx , ubyte arg1, ubyte arg2, ubyte arg3, ubyte arg4 )
{
	ubyte serialCmd [ 6 ] = { 0,0,0,0,0,0 };		// fully valid serial command for hardware
	ubyte serialSize = 2;							// number of bytes in the serial command
	
					  
	// put the command header and command identifier at the top of the serial command
					  
	serialCmd [0] = CmdHeader;
	serialCmd [1] = cmdIdx;
	
	
	// compute the number of additional argument bytes based on command type (and validate command)

	switch ( cmdIdx )
	{
		
		// 0 byte arguments
			
		case ClearMixersAB         :					// 0 | sets all mixer and oscillator values to zero
		case RampToTargetsAB       :					// 0 | calculates ramp values and activate ramp and target options
		case SetVoiceDefaults      :					// 0 | sets voice values to defaults
		case TurnQOn               :					// 0 | sets the Q output pin high
		case TurnQOff              :					// 0 | sets the Q output pin low
		case ClearMixerAndOsc_A    :					// 0 | sets mixer and oscillator values to zero
		case ClearMixerAndOsc_B    :					// 0 | sets mixer and oscillator values to zero
		case RampToTargets_A       :					// 0 | calculates ramp values and activate ramp and target options
		case RampToTargets_B       :					// 0 | calculates ramp values and activate ramp and target options 
		case TriggerOsc_A1         :					// 0 | start the envelope for osc 1
		case TriggerOsc_B1         :					// 0 | start the envelope for osc 1
		case TriggerOsc_A2         :					// 0 | start the envelope for osc 2
		case TriggerOsc_B2         :					// 0 | start the envelope for osc 2
		case TriggerOsc_A3         :					// 0 | start the envelope for osc 3
		case TriggerOsc_B3         :					// 0 | start the envelope for osc 3
		case TriggerMixer_A        :					// 0 | start the mixer envelope
		case TriggerMixer_B        :					// 0 | start the mixer envelope
		case ReleaseOsc_A1         :					// 0 | release the envelope for osc 1
		case ReleaseOsc_B1         :					// 0 | release the envelope for osc 1
		case ReleaseOsc_A2         :					// 0 | release the envelope for osc 2
		case ReleaseOsc_B2         :					// 0 | release the envelope for osc 2
		case ReleaseOsc_A3         :					// 0 | release the envelope for osc 3
		case ReleaseOsc_B3         :					// 0 | release the envelope for osc 3
		case ReleaseMixer_A        :					// 0 | release the mixer envelope
		case ReleaseMixer_B        :					// 0 | release the mixer envelope
		case ResetToVoice_A        :					// 0 | resets mixer to voice parameters
		case ResetToVoice_B        :					// 0 | resets mixer to voice parameters
			
			break;										// 0 additional bytes in the serial command
			
			
			
		// 1 byte arguments
			
		case ReadOneByte           :					// 1 | addr               | polls 1 byte at register address
		case SetVoiceNote          :					// 1 | 1 byte note value  | sets voice frequency to musical note val
		case SetVoiceDelay         :					// 1 | 1 byte delay       | set voice delay to specified value
		case LoadNoteOsc_A1        :					// 1 | 1 byte note value  | loads note value for osc 1
		case LoadNoteOsc_B1        :					// 1 | 1 byte note value  | loads note value for osc 1
		case LoadNoteOsc_A2        :					// 1 | 1 byte note value  | loads note value for osc 2
		case LoadNoteOsc_B2        :					// 1 | 1 byte note value  | loads note value for osc 2
		case LoadNoteOsc_A3        :					// 1 | 1 byte note value  | loads note value for osc 3
		case LoadNoteOsc_B3        :					// 1 | 1 byte note value  | loads note value for osc 3
		case LoadSoundMixer_A      :					// 1 | 1 byte sound fx    | loads sound effect into mixer/oscillators
		case LoadSoundMixer_B      :					// 1 | 1 byte sound fx    | loads sound effect into mixer/oscillators
		case LoadPlayNoteOsc_A1    :					// 1 | 1 byte note value  | loads the note and starts env osc 1
		case LoadPlayNoteOsc_B1    :					// 1 | 1 byte note value  | loads the note and starts env osc 1
		case LoadPlayNoteOsc_A2    :					// 1 | 1 byte note value  | loads the note and starts env osc 2
		case LoadPlayNoteOsc_B2    :					// 1 | 1 byte note value  | loads the note and starts env osc 2
		case LoadPlayNoteOsc_A3    :					// 1 | 1 byte note value  | loads the note and starts env osc 3
		case LoadPlayNoteOsc_B3    :					// 1 | 1 byte note value  | loads the note and starts env osc 3
			
			serialSize += 1;							// 1 additional byte in the serial command
			serialCmd [2] = arg1;						// copy byte 0 from argument list
			break;
			
			
		// 2 byte arguments
			
		case WriteOneByte          :					// 2 | addr, 1 data byte  | writes 1 byte to register address
		case SetVoiceFrequency     :					// 2 | 2 bytes frequency  | sets voice frequency to secified value 
		case LoadFreqOsc_A1        :					// 2 | 2 bytes frequency  | loads specified frequency into osc 1
		case LoadFreqOsc_B1        :					// 2 | 2 bytes frequency  | loads specified frequency into osc 1
		case LoadFreqOsc_A2        :					// 2 | 2 bytes frequency  | loads specified ferquency into osc 2
		case LoadFreqOsc_B2        :					// 2 | 2 bytes frequency  | loads specified ferquency into osc 2
		case LoadFreqOsc_A3        :					// 2 | 2 bytes frequency  | loads specified frequency into osc 3  
		case LoadFreqOsc_B3        :					// 2 | 2 bytes frequency  | loads specified frequency into osc 3
		case LoadPlayFreqOsc_A1    :					// 2 | 2 bytes frequency  | loads the frequency and starts env osc 1
		case LoadPlayFreq_B1       :					// 2 | 2 bytes frequency  | loads the frequency and starts env osc 1
		case LoadPlayFreqOsc_A2    :					// 2 | 2 bytes frequency  | loads the frequency and starts env osc 2
		case LoadPlayFreq_B2       :					// 2 | 2 bytes frequency  | loads the frequency and starts env osc 2
		case LoadPlayFreqOsc_A3    :					// 2 | 2 bytes frequency  | loads the frequency and starts env osc 3
		case LoadPlayFreq_B3       :					// 2 | 2 bytes frequency  | loads the frequency and starts env osc 3
		
			serialSize += 2;							// 2 additional bytes in the serial command
			serialCmd [2] = arg1;						// copy byte 0 from argument list
			serialCmd [3] = arg2;						// copy byte 1 from argument list

			break;
		
			
		// 3 byte arguments
			
		case WriteTwoBytes         :					// 3 | addr, 2 data bytes     | writes two bytes to register address
			serialSize += 3;							// 3 additional bytes in the serial command
			serialCmd [2] = arg1;						// copy byte 0 from argument list
			serialCmd [3] = arg2;						// copy byte 1 from argument list
			serialCmd [4] = arg3;						// copy byte 2 from argument list
			break;

		case WriteOneByteWithMask  :					// 3 | addr, mask , data byte  | writes 1 masked byte to register address

			serialSize += 3;							// 3 additional bytes in the serial command
			serialCmd [2] = arg1;						// copy byte 0 from argument list
			serialCmd [3] = arg3;						// copy byte 1 from argument list
			serialCmd [4] = arg2;						// copy byte 2 from argument list
			break;
			
			
		// 4 byte arguments
			
		case WriteThreeBytes       :					// 4 | addr, 3 data bytes     | writes three bytes to register address
			
			serialSize += 4;							// 4 additional bytes in the serial command
			serialCmd [2] = arg1;						// copy byte 0 from argument list
			serialCmd [3] = arg2;						// copy byte 1 from argument list
			serialCmd [4] = arg3;						// copy byte 2 from argument list
			serialCmd [5] = arg4;						// copy byte 3 from argument list
			break;
			
		default:
			return;										// command is not one of the valid types
			
	}
			
		
	// send the serial command out to the interface one byte at a time
	
	
	for ( int serialIdx = 0; serialIdx < serialSize; serialIdx++ )
		serialInt->write  ( serialCmd [serialIdx] );
}					  

						
// ---------------------------------------------------------------------------------------------------------------------------------------------
// sendCommand - send a specific low level command ( add associated data ) to the sound chip
// ---------------------------------------------------------------------------------------------------------------------------------------------

void  GinSing::writeRegister ( GSRegister regIdx , ubyte value , ubyte mask )
{
	gsPtr->sendCommand ( WriteOneByteWithMask , regIdx , value , mask );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// sendCommand - send a specific low level command ( add associated data ) to the sound chip
// ---------------------------------------------------------------------------------------------------------------------------------------------

ubyte  GinSing::readRegister  ( GSRegister regIdx )
{
	// flush out any old data
	
	serialInt->flush();
	
	
	// send a command requesting to get one byte back
	
	gsPtr->sendCommand ( ReadOneByte , regIdx );
	
	
	// wait for the data to come back
	
	ubyte readByte;
	while ( ! serialInt->readByte( & readByte ) );
	
	return ( readByte );	
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// _dumpRegisters - prints the current value of all the registers ( be sure to call Serial.begin() before calling this! )
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _dumpRegisters()
{
	// read the contents of all the registers

	for ( int regIdx = (int) A_MixControl_0; regIdx <= (int) Output_B3; regIdx++ )
	{
		ubyte value = gsPtr->readRegister ( (GSRegister) regIdx );
	
		char msgOut[60];
		sprintf ( msgOut , "reg[%.3d] = [%.3d (0x%.2x)]" , regIdx , value , value );
		Serial.println ( msgOut );
	}
}

