#ifndef _GINSING_DEF
#define _GINSING_DEF


// ---------------------------------------------------------------------------------------------------------------------------------------------
// GinSing - a sound and voice synthesizer library for the Arduino utilizing the SoundGin chip
// ---------------------------------------------------------------------------------------------------------------------------------------------
// 
// GinsSing is a software library that provides an API to the Savage Industries SoundGin chip on the Arduino development platform. The chip is a 
// complex sound synthesizer that can be used as a 6 channel waveform synth, a polyophonic musical instrument, a sound effect generator, a voice 
// synthesizer, and more. For more information please visit http://www.soundginsound.com.
//
//
// This work is provided under the terms of the Creative Commons Attribution-NonCommercial 3.0 Unsupported ( "CCPL" ). The work is protected by 
// copyright and/or other applicable law. Any use of this work other than as authorized under the CCPL license or copyright law is prohibited. 
// You are free to use this code for non-commercial applications, and are free to make modifications to it as you see fit as long as you 
// credit the original author ( www.engeldinger.com ). In order to improve the work and help provide support to others, we request that you visit
// our website ( http://www.soundginsound.com ) and notify us of bugs, feature enhancements, and how are are using this code. By doing this,
// we can maintain the code and redistribute updated versions to the community.
//
// ---------------------------------------------------------------------------------------------------------------------------------------------
// About the Library:
//
// This GinSing library is designed for ease of use and removes the complexity involved in talking directly to the chip. The library creates
// 4 basic operating modes that each have their own interface:
//
//   preset mode		allows the triggering of the 31 preset sound effects ( up to two simultaneously )
//   poly mode			creates a 6 channel polyphonic synthesizer useful for creating musical melodies
//   synth mode			creates a 2 bank ( 3 oscillator per bank ) waveform synthesizer capable of all kinds of modulation and patching
//   voice mode			allows the use of the chip to reproduce morphed phonemes to create smooth human speech simulation
//
// The GinSing library uses various header files to define the functional interfaces. It is recommended that you take a look at these files
// to get an understanding of the API as well as simple examples on how to use it:
//
//   GinSingDefs.h		contains the constants that are used throughout the library. They are located in one place for coding convenience
//   GinSingPreset.h	preset mode interface
//	 GinSingPoly.h		poly mode interface
//   GinSingSynth.h		synth mode interface
//   GinSingVoice.h		voice mode interface
//   GinSingMaster.h	master control functions (and output mixer)
//
//
// ---------------------------------------------------------------------------------------------------------------------------------------------
//  Source Example:
//
// Although the interface functions vary over different modes, the mechanism for creating and using the library is the same. This source code
// example shows you the basic calls that will allow you to start up GinSing and preview each of the available modes:
//
// #include <GinSing.h>									// this is the only include file you need in your code
//
//   GinSing  GS;										// create the class library from which all methods and data are accessed
//
//   GS.begin();										// start up the library and the chip
//
//    GinSingPoly *poly = GS.getPoly();					// get the poly mode functions
//    poly->begin();									// switch into poly mode
//    poly->preview();									// preview the mode
//
//    GinSingPreset *preset = GS.getPreset();			// get the preset mode functions
//    preset->begin();									// switch into preset mode
//    preset->preview();								// preview the mode
//
//    GinSingPoly *poly = GS.getPoly();					// get the poly mode functions
//    poly->begin();									// switch into poly mode
//    poly->preview();									// preview the mode

//    GinSingSynth *synth = GS.getSynth();				// get the synth mode functions
//    synth->begin();									// switch into synth mode
//    synth->preview();									// preview themode
//
//  GS.end();											// showdown the library and the chip
//
// ---------------------------------------------------------------------------------------------------------------------------------------------

#if defined(ARDUINO) && ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#define  GINSING_VERSION  4.0													// current version of this library


// global commonly used variable types

typedef unsigned char ubyte;													// single unsigned byte ( 0 to 255 )
typedef signed   char sbyte;													// single signed byte  ( -127 to 127 )


#include <GinSingDefs.h>														// common variable definitions and constants
#include <GinSingPoly.h>														// polyphonic instrument mode
#include <GinSingPreset.h>														// preset effect mode
#include <GinSingVoice.h>														// voice synthesis mode
#include <GinSingSynth.h>														// waveform synthesizer mode
#include <GinSingMaster.h>														// master functions
#include <GinSingSerial.h>														// serial interface functions

// ---------------------------------------------------------------------------------------------------------------------------------------------
// GinSing -  the main library class - the only class that needs to be instanced
// ---------------------------------------------------------------------------------------------------------------------------------------------


class GinSing
{
		
public:

		
	// general methods
		
	GinSing();  ~GinSing();														// constructor / destructor
			
	void begin   ( int recvPin , int xmitPin , int ovfPin );					// connect and initialize device
	void end     ();															// disconnect from the device
	void reset   ();															// reset device to ground level state
	bool isReady ();															// true if ready to accept, false if serial line is busy
	int  getVersion () { return ( GINSING_VERSION ); }							// get the version of this library
	
	
	// accessor classes
	
	GinSingPoly   * getPoly   () { return ( & poly );	}						// get the poly mode functions
	GinSingVoice  * getVoice  () { return ( & voice );  }						// get the voice mode functions
	GinSingPreset * getPreset () { return ( & preset );	}						// get the preset mode functions
	GinSingSynth  * getSynth  () { return ( & synth );  }						// get the synth mode functions
	GinSingMaster * getMaster () { return ( & master ); }						// get the master functions
	GinSingSerial * getSerial () { return ( serialInt ); }						// get the serial interface

	
	
	// low level system methods that are used to communicate with the chip. these are only needed if you wish to communicate directly
	// to the chip, thusly voiding any of the above oeprationg modes
	
	void sendCommand ( GSCommand cmdIdx  ,										// command to execute ( see GinSingDefs.h for available commands )
						  ubyte arg1 = 0x0 ,									// argument 1 ( optional depending on command used )
						  ubyte arg2 = 0x0 ,									// argument 2 ( optional depending on command used )
						  ubyte arg3 = 0x0 ,									// argument 3 ( optional depending on command used )
						  ubyte arg4 = 0x0 );									// argument 4 ( optional depending on command used )
	
	
	void  writeRegister ( GSRegister regIdx , ubyte value , ubyte mask );		// write a value to specified register masked by the mask bits
	ubyte readRegister  ( GSRegister regIdx );									// read the value of the specified register
	
	
private:
		
	GinSingSerial *      serialInt;												// serial interface class
	GinSingPoly          poly;													// polyphonic mode functions
	GinSingVoice         voice;													// voice mode functions
	GinSingPreset        preset;												// preset effect mode functions
	GinSingSynth         synth;													// synth mode funtions
	GinSingMaster        master;												// output mixer and global config
	
	
};

extern GinSing * gsPtr;															// global pointer to the static instance 

#endif // _GINSING_DEF

