// ---------------------------------------------------------------------------------------------------------------------------------------------
// GinSingMaster - the component of the GinSing system that handles master functions
// ---------------------------------------------------------------------------------------------------------------------------------------------

#include "GinSing.h"


#define MixerMaxCount 2


// mixer commands

static GSCommand  _triggerCmds[] = { TriggerMixer_A , TriggerMixer_B };
static GSCommand  _releaseCmds[] = { ReleaseMixer_A , ReleaseMixer_B };


// mixer registers

static GSRegister _ampRegs  [] = { A_Amplitude       , B_Amplitude       };
static GSRegister _envRegs  [] = { A_EnvelopeControl , B_EnvelopeControl };
static GSRegister _atkRegs  [] = { A_EnvelopeAttack  , B_EnvelopeAttack  };
static GSRegister _dcyRegs  [] = { A_EnvelopeDecay   , B_EnvelopeDecay   };
static GSRegister _rlsRegs  [] = { A_EnvelopeRelease , B_EnvelopeRelease };
static GSRegister _atxRegs  [] = { A_AmplitudeXLow   , B_AmplitudeXLow   };
static GSRegister _amptRegs [] = { A_AmplitudeTarget , B_AmplitudeTarget };


// global registers

static GSRegister _bndRegs  [] = { PitchBend_A       , PitchBend_B       };


// global control registers

#define GC_UnusedLow            0x07									// [0-2] no function
#define GC_FlowControl          0x08									// Xon / Xoff ( preset and not cofigurable )
#define GC_UserOutput           0x10									// [4  ] user output pin
#define GC_SpeedLow             0x20									// [5  ] slow down clock speed
#define GC_UnusedHi				0xc0									// [6-7] no function


// mixer envelope control bits

#define EC_StatusBits          0x03										// [0-1] read only - envelope phase
#define EC_FrqTgtCtrlBits      0x04										// [2  ] frq tgt bit | bit 4 frq ramp bit   ( need ramp bit as well )
#define EC_AmpTgtCtrlBits      0x08										// [3  ] enable or disable amplitude target ( need ramp bit as well )
#define EC_FrqRampCtlBits      0x10										// [4  ] enable or disable frequency ramp
#define EC_AmpRampCtrlBits     0x20										// [5  ] bit 5 (AMPRAMP)
#define EC_RunningBits         0x40										// [6  ] read only - envelope running
#define EC_ActiveBits          0x80										// [7  ] read only - envelpe active


// helper functions

void _setAllMixCmd0         ( GSCommand *cmds );

void _setAllMixReg1Byte     ( GSRegister *regs , ubyte value );
void _setAllMixReg1ByteMask ( GSRegister *regs , ubyte value , ubyte mask );
void _setAllMixReg2Bytes    ( GSRegister *regs , ubyte val1, ubyte val2 );


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setMode - configure the master settings for the specified mode
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::reset()
{
	// set master register values that are not accessable through the API
	
	gsPtr->sendCommand ( WriteOneByte , MiscellaneousControl , GC_FlowControl  );				// enable flow control
	
	
	// send out a reset to all registers 0-127 ( mixers A/B and oscillators 1-6 will contain zeroes )
			
	gsPtr->sendCommand ( ClearMixersAB );			
			
	
	// set the default master properties
	
	gsPtr->getSynth()->release ( OSC_ALL );														// release all the oscillators
	
	setMasterAmplitude ( 1.0f );																// master amplitude set to 100%
	setAmplitude       ( MIX_ALL , 1.00f );														// both mixers set to 50%
	setEnvelope        ( MIX_ALL , AT_2MS , 1.0f , DR_6MS , 1.0f , DR_6MS , 0.0f );				// mixer envelopes minimum tail
	setBendFreq        ( MIX_ALL , 0.0f );														// zero bend frequency
	
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmplitude - set the master output amplitude
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::setMasterAmplitude( float amplitude )
{
	// convert the amplitude into a signed byte
	
	ubyte ampByte = (ubyte)( 127.0f * amplitude );
	setMasterAmplitudeVal ( ampByte );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmplitudeVal - set the master output amplitude bu value
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::setMasterAmplitudeVal ( ubyte ampVal )
{
	// set the master amplitude register
	
	gsPtr->sendCommand ( WriteOneByte , MasterAmplitude , ampVal );
}
// ---------------------------------------------------------------------------------------------------------------------------------------------
// enableUserOutput - enable or disable user controlled output on the chip
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::enableUserOutput( bool enable )
{
	// set or reset the Q output bit on the miscellanous control register
	
	if ( enable ) gsPtr->sendCommand ( WriteOneByteWithMask , MiscellaneousControl , GC_UserOutput , GC_UserOutput );
	else          gsPtr->sendCommand ( WriteOneByteWithMask , MiscellaneousControl , 0x0           , GC_UserOutput );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// enableSlowClock - enable or disable slowdown of the system clock
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::enableSlowClock( bool enable )
{
	// set or reset slow clock bit on the miscellanous control register
	
	if ( enable ) gsPtr->sendCommand ( WriteOneByteWithMask , MiscellaneousControl , GC_SpeedLow , GC_SpeedLow );
	else          gsPtr->sendCommand ( WriteOneByteWithMask , MiscellaneousControl , 0x0         , GC_SpeedLow );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmplitude - set a mixer output amplitude
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::setAmplitude( GSMasterMixer mixIdx , float amplitude )
{
	// convert amplitude to a signed byte
	
	sbyte ampByte = (sbyte)( 127.0f * amplitude );
	setAmplitudeVal ( mixIdx , ampByte );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmplitudeVal - set a mixer output amplitude by value
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::setAmplitudeVal( GSMasterMixer mixIdx , ubyte ampVal )
{
	
	// set the mixer amplitude register(s)
	
	if ( mixIdx != MIX_ALL ) gsPtr->sendCommand ( WriteOneByte , _ampRegs [ mixIdx ] , ampVal );
	else                     _setAllMixReg1Byte (                _ampRegs            , ampVal );
	
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// trigger - trigger the ampltiude envelope for a voice
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::trigger ( GSMasterMixer mixIdx )
{	
	// send triger command(s)
	
	if ( mixIdx != MIX_ALL ) gsPtr->sendCommand ( _triggerCmds [ mixIdx ] );
    else                     _setAllMixCmd0     ( _triggerCmds );
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// release - release the ampltiude envelope
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::release ( GSMasterMixer mixIdx )
{	
	// send the release command(s)
	
	if ( mixIdx != MIX_ALL ) gsPtr->sendCommand ( _releaseCmds [ mixIdx ] );
    else                     _setAllMixCmd0     ( _releaseCmds );
	
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setEnvelope - set the ADSR envelope parameters
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::setEnvelope  ( GSMasterMixer mixIdx ,
								   GSAttackDur attackDur  , float attackAmp  ,
								   GSDecRelDur decayDur   , float decayAmp   ,
								   GSDecRelDur releaseDur , float releaseAmp )
{	
	
	// construct ADR bytes ( hi 4 bytes amplitude, low 4 bytes duration )
	
	ubyte atkByte = ( (ubyte) ( 0x0f * attackAmp   ) << 4 ) + attackDur;
	ubyte dcyByte = ( (ubyte) ( 0x0f * decayAmp    ) << 4 ) + decayDur;
	ubyte rlsByte = ( (ubyte) ( 0x0f * releaseAmp  ) << 4 ) + releaseDur;
	
	setEnvelopeVal  (  mixIdx ,
					  attackDur  ,  atkByte  ,
					  decayDur   ,  dcyByte  ,
					  releaseDur ,  rlsByte );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setEnvelope - set the ADSR envelope parameters
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::setEnvelopeVal  ( GSMasterMixer mixIdx ,
								      GSAttackDur attackDur  , ubyte attackAmpVal  ,
								      GSDecRelDur decayDur   , ubyte decayAmpVal   ,
								      GSDecRelDur releaseDur , ubyte releaseAmpVal )
{	
	
	// set the ADSR register(s)
	
	if ( mixIdx != OSC_ALL )
	{
		gsPtr->sendCommand ( WriteOneByte , _atkRegs [ mixIdx ] , attackAmpVal );
		gsPtr->sendCommand ( WriteOneByte , _dcyRegs [ mixIdx ] , decayAmpVal );
		gsPtr->sendCommand ( WriteOneByte , _rlsRegs [ mixIdx ] , releaseAmpVal );
	}
	else
	{
		_setAllMixReg1Byte ( _atkRegs , attackAmpVal );
		_setAllMixReg1Byte ( _dcyRegs , decayAmpVal );
		_setAllMixReg1Byte ( _rlsRegs , releaseAmpVal );
	}
	
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// enableAmpTarget - enable or disable amplitude targeting
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::enableAmpTarget  ( GSMasterMixer mixIdx , bool enable )
{
	
	
	// set the envelope control bits
		
	if ( mixIdx != MIX_ALL ) 
	{
		if ( enable )
			gsPtr->sendCommand ( WriteOneByteWithMask , _envRegs [ mixIdx ] , EC_AmpTgtCtrlBits , EC_AmpTgtCtrlBits );
		else
			gsPtr->sendCommand ( WriteOneByteWithMask , _envRegs [ mixIdx ] , 0x0               , EC_AmpTgtCtrlBits );
	}
	else
	{
		if ( enable )
			_setAllMixReg1ByteMask ( _envRegs  , EC_AmpTgtCtrlBits , EC_AmpTgtCtrlBits );
		else
			_setAllMixReg1ByteMask ( _envRegs  , 0x0               , EC_AmpTgtCtrlBits );
	}
	
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmpTarget - set an amplitude target and the rate at which it will track towards it
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingMaster::setAmpTarget  ( GSMasterMixer mixIdx , float ampTgt , float relRate )
{
	
	// convert target amplitude to 1 byte value
	
	uint ampByte = (uint) ( 127.0f * ampTgt  );
	
	// convert target rate to 2 byte value
	
	unsigned long xrInt = (unsigned long) ( 0x7fff * relRate );

	setAmpTargetVal ( mixIdx , ampByte , xrInt );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmpTargetVal - set an amplitude target with an integer value
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingMaster::setAmpTargetVal  ( GSMasterMixer mixIdx , ubyte ampTgtVal , uint relRateVal )
{
	
	// convert target rate to 2 byte value
	
	ubyte xrLo   = (ubyte)( ( relRateVal & 0x0000ff ) );
	ubyte xrHi   = (ubyte)( ( relRateVal & 0x00ff00 ) >> 8 );
	
	
	// send the target, rate, and amplitude bits
	
	if ( mixIdx != MIX_ALL ) 
	{
		gsPtr->sendCommand ( WriteOneByte  , _amptRegs[ mixIdx ] , ampTgtVal );
		gsPtr->sendCommand ( WriteTwoBytes , _atxRegs [ mixIdx ] , xrLo , xrHi );
	}
	else
	{
		_setAllMixReg1Byte  ( _amptRegs , ampTgtVal   );
		_setAllMixReg2Bytes ( _atxRegs  , xrLo , xrHi );
	}	
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// enableAmpRamp - enable or disable amplitude ramping
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingMaster::enableAmpRamp  ( GSMasterMixer mixIdx , bool enable )
{	

	// set the transition rate and ramp control bit
		
	if ( mixIdx != MIX_ALL ) 
	{
		if ( enable )
			gsPtr->sendCommand ( WriteOneByteWithMask , _envRegs [ mixIdx ] , EC_AmpRampCtrlBits , EC_AmpRampCtrlBits );
		else
			gsPtr->sendCommand ( WriteOneByteWithMask , _envRegs [ mixIdx ] , 0x0                , EC_AmpRampCtrlBits );
				
	}
	else
	{
		if ( enable )
			_setAllMixReg1ByteMask ( _envRegs , EC_AmpRampCtrlBits , EC_AmpRampCtrlBits );
		else
			_setAllMixReg1ByteMask ( _envRegs , 0x0                , EC_AmpRampCtrlBits );
	}
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmpRamp - set an ampltiude ramp sweep at a given rate
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingMaster::setAmpRamp  ( GSMasterMixer mixIdx , float relRate )
{	

	// convert to a 2 byte value
	
	uint xrInt = (uint) ( 0x7fff * relRate );
	
	setAmpRampVal ( mixIdx , xrInt );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmpRampVal - set an ampltiude ramp using integers
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingMaster::setAmpRampVal  ( GSMasterMixer mixIdx , uint relRateVal )
{	
	
	// convert rate to two bytes
	
	ubyte xrLo   = (ubyte)( ( relRateVal & 0x00ff ) );
	ubyte xrHi   = (ubyte)( ( relRateVal & 0xff00 ) >> 8 );
		
		
	// set the transition rate
		
	if ( mixIdx != MIX_ALL ) 
	{
		gsPtr->sendCommand ( WriteTwoBytes , _atxRegs [ mixIdx ] , xrLo , xrHi );
	}
	else
	{
		_setAllMixReg2Bytes    ( _atxRegs , xrLo , xrHi );
	}
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setBendFreq - set a bend frquency that is applied per oscillator ( as per synth oscillator setting )
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingMaster::setBendFreq( GSMasterMixer mixIdx , float freqHz )
{
	// compute the bend frequency as an integer
	
	ubyte freqInt = (ubyte) ( freqHz / 0.0009313226 );
	setBendFreqVal ( mixIdx , freqInt );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setBendFreqVal - set bend frequency by integer
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingMaster::setBendFreqVal ( GSMasterMixer mixIdx , ubyte freqVal )
{
	
	// only high byte is used for frequency on bend
	
	ubyte freqHi = (ubyte)( ( freqVal & 0xff0000 ) >> 16 );	
	
	
	// set the bend frequency register(s)
	
	if ( mixIdx != MIX_ALL )  gsPtr->sendCommand  ( WriteOneByte , _bndRegs [ mixIdx ] , freqHi );
	else                       _setAllMixReg1Byte (                _bndRegs ,            freqHi );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// _setAllMixCmd0 - send all mixers a non-argument command
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllMixCmd0 ( GSCommand *cmds )
{	
	for ( int loopIdx = 0; loopIdx < MixerMaxCount; loopIdx++ ) 
		gsPtr->sendCommand ( cmds [ loopIdx ] );
}	

// ---------------------------------------------------------------------------------------------------------------------------------------------
// _SetAllMixReg1Byte - set all mixer registers to the same 1 byte value
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllMixReg1Byte ( GSRegister *regs , ubyte value )
{	
	for ( int loopIdx = 0; loopIdx < MixerMaxCount; loopIdx++ )
		gsPtr->sendCommand ( WriteOneByte , regs [ loopIdx ] , value );
}
// ---------------------------------------------------------------------------------------------------------------------------------------------
// _setAllMix1ByteMask - send all mixer registers with the same 1 byte masked value
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllMixReg1ByteMask ( GSRegister *regs , ubyte value , ubyte mask )
{
	for ( int loopIdx = 0; loopIdx < MixerMaxCount; loopIdx++ )
		gsPtr->sendCommand ( WriteOneByteWithMask , regs [ loopIdx ] , value , mask );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// _SetAllMixReg2Bytes - set all mixer registers to the same 2 byte value
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllMixReg2Bytes ( GSRegister *regs , ubyte val1, ubyte val2 )
{	
	for ( int loopIdx = 0; loopIdx < MixerMaxCount; loopIdx++ )
		gsPtr->sendCommand ( WriteTwoBytes , regs [ loopIdx ] , val1, val2 );
}	

