// ---------------------------------------------------------------------------------------------------------------------------------------------
// GinSingPoly - polyphonic instrument support for the Arduino GinSing shield
// ---------------------------------------------------------------------------------------------------------------------------------------------


#include "GinSing.h"


float _portamentoRate[GinSingPoly::NumVoices] = { 0.0f, 0.0f , 0.0f , 0.0f , 0.0f , 0.0f };		// non-zero value will cause note blending
float _portamentoAmp [GinSingPoly::NumVoices] = { 1.0f, 1.0f , 1.0f , 1.0f , 1.0f , 1.0f };		// amplitudes used during portamento (no ADSR)

float _portamentoRateAll         = 0.0f;														// when applied to all voices
float _portamentoAmpAll          = 1.0f;														// when applied to all voices

// table of zeroth octave frequencies
// note: these match the chip and not an equally tempered scale ( n+1 = n * ( 1 + 2^(1/12) )

float BaseNoteFrq[] = { 16.35159375f ,            // C_0
                        17.32390625f ,            // CS_0
                        18.35403906f ,            // D_0
                        19.44542969f ,            // DS_0
                        20.60171875f ,            // E_0
                        21.82675781f ,            // F_0
                        23.12464844f ,            // FS_0
                        24.49971094f ,            // G_0
                        25.95653906f ,            // GS_0
                        27.50000000f ,            // A_0
                        29.13523438f ,            // AS_0
                        30.86771094f ,            // B_0
};


// ---------------------------------------------------------------------------------------------------------------------------------------------
// begin - configure the system for polyphonic instrument mode
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingPoly::begin()
{
	
	// configure master volume
	
	GinSingMaster *m = gsPtr->getMaster();		
	
	m->reset();																					// reset to master defaults
	m->setAmplitude ( MIX_ALL , 0.3f );														   // set both mixer volumes to 30%
	
	
	// configure the synth for 6 note polyphonic
	
	GinSingSynth *s = gsPtr->getSynth();
	
	s->selectBank  ( BANK_A );																	// select bank A
	s->setPatch    ( OSC_1_TO_MIXER | OSC_2_TO_MIXER | OSC_3_TO_MIXER );						// send all oscillators to mix
	s->enable      ( OSC_ALL , true );															// enable all oscillators in bank
	
	s->selectBank  ( BANK_B );																	// select bank B
	s->setPatch    ( OSC_1_TO_MIXER | OSC_2_TO_MIXER | OSC_3_TO_MIXER );						// send all oscillators to mix
	s->enable      ( OSC_ALL , true );															// enable all oscillators in bank

	
	// set up the default properties for the oscillators
	
	setEnvelope   ( AllVoices , AT_24MS , 0.5f , DR_2MS , 0.5f , DR_575MS , 0.0f );			    // 30% volume 100ms rise, 0.5s fall
	setNote       ( 0 , A_3  );																	// 236,223 = A3	
	setNote       ( 1 , CS_4 );																	// 297,623 = C#4
	setNote       ( 2 , E_4  );																	// 353,934 = E4
	setNote       ( 3 , A_4  );																	// 472,446 = A4
	setNote       ( 4 , CS_5 );																	// 595,244 = C#5
	setNote       ( 5 , E_5  );																	// 707,870 = E5
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// preview - preview poly mode
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingPoly::preview()
{
	
	// trigger the default voices 
	
	for ( ubyte voiceIdx = 0; voiceIdx < GinSingPoly::NumVoices; voiceIdx++ ) { trigger ( voiceIdx ); delay ( 150 ); }
	delay ( 1000 );
	
	
	// release all the voices and wait for release
	
	gsPtr->getPoly()->release ( AllVoices );
	delay ( 300 );
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// trigger - trigger the ampltiude envelope for a voice
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingPoly::trigger ( ubyte voiceIdx )
{
	float portVal;

	// if using portamento trigger by setting volume
	
	if ( voiceIdx == GinSingPoly::AllVoices ) portVal = _portamentoRateAll;
	else                                      portVal = _portamentoRate [ voiceIdx ];
	
	if ( portVal != 0.0f )
	{
		float amplitude;
		
		if ( voiceIdx == GinSingPoly::AllVoices ) amplitude = _portamentoAmpAll;
		else                                      amplitude = _portamentoAmp [ voiceIdx ];		
		
		gsPtr->getSynth()->setAmplitude ( (GSSynthOsc) voiceIdx , amplitude );
	}
	
	
	// otherwise use synth mode trigger

	else
	{
		gsPtr->getSynth()->trigger ( (GSSynthOsc) voiceIdx );
	}
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// release - release the ampltiude envelope
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingPoly::release ( ubyte voiceIdx )
{
	float portVal;

	// if using portamento release by disabling volume
	
	if ( voiceIdx == GinSingPoly::AllVoices ) portVal = _portamentoRateAll;
	else                                      portVal = _portamentoRate [ voiceIdx ];
	
	if ( portVal != 0.0f )
	{
		gsPtr->getSynth()->setAmplitude ( (GSSynthOsc) voiceIdx , 0.0f );
	}
	
	// otherwise use synth mode release

	else
	{
		gsPtr->getSynth()->release ( (GSSynthOsc) voiceIdx );
	}
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setNote - set the musical note for a voice
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingPoly::setNote ( ubyte voiceIdx , GSNote noteIdx )
{
	float portVal;
	
	// if using portamento set target frequency rather than note
	
	if ( voiceIdx == GinSingPoly::AllVoices ) portVal = _portamentoRateAll;
	else                                      portVal = _portamentoRate [ voiceIdx ];
	
	if ( portVal != 0.0f )
	{
		// convert note to frequency and set as target
		
		ubyte octave    = noteIdx / 16;
		ubyte semitone  = noteIdx % 16;
		ubyte octavePow = 1 << octave;
			
		float noteFrq = (float) octavePow * BaseNoteFrq [ semitone ];
		gsPtr->getSynth()->setFreqTarget ( (GSSynthOsc) voiceIdx,  noteFrq , portVal );
	}
	else
	{
		// otherwise use synth mode set note
	
		gsPtr->getSynth()->setNote ( (GSSynthOsc) voiceIdx , noteIdx );
	}
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setWaveform - set the output waveform
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingPoly::setWaveform  ( ubyte voiceIdx , GSWaveType waveform )
{
	// use synth mode set waveform
	
	gsPtr->getSynth()->setWaveform ( (GSSynthOsc) voiceIdx , waveform );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setDistorion - set the waveform distirtion
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingPoly::setFreqDist ( ubyte voiceIdx , float distLevel )
{	
	// use synth mode to set frequency distortion
	
	gsPtr->getSynth()->setFreqDist ( (GSSynthOsc) voiceIdx , distLevel );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setDutyCycle - set the waveform duty cycle ( pulse wave only )
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingPoly::setDutyCycle ( ubyte voiceIdx , float dutyCycle )
{
	// use synth mode to set duty cycle
	
	gsPtr->getSynth()->setDutyCycle ( (GSSynthOsc) voiceIdx , dutyCycle );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setEnvelope - set the ADSR envelope parameters
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingPoly::setEnvelope  ( ubyte voiceIdx                            ,
								 GSAttackDur attackDur  , float attackAmp  ,
				                 GSDecRelDur decayDur   , float decayAmp   ,
				                 GSDecRelDur releaseDur , float releaseAmp )
{
	// use synth mode set envelope
	
	gsPtr->getSynth()->setEnvelope ( (GSSynthOsc) voiceIdx , attackDur, attackAmp , decayDur , decayAmp , releaseDur , releaseAmp );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setPortamento - set the notes to bend at the specified rate
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingPoly::setPortamento ( ubyte voiceIdx , float amplitude , float relRate )
{
	float portRate;
	
	// set the rate and volume for subsequent note triggers
						 
	if ( voiceIdx == GinSingPoly::AllVoices ) 
	{
		for ( int v = 0; v < GinSingPoly::NumVoices; v++ ) _portamentoRate [v] = relRate;
		for ( int v = 0; v < GinSingPoly::NumVoices; v++ ) _portamentoAmp  [v] = amplitude;
		_portamentoRateAll = relRate;
		_portamentoAmpAll  = amplitude;
	}
	else
	{
		_portamentoRate [ voiceIdx ] = relRate;
		_portamentoAmp  [ voiceIdx ] = amplitude;
	}
	
						 
	// if rate is zero disable appropriately - otheriwise set initi value
						 
	if ( relRate == 0.0f ) 
	{
		gsPtr->getSynth()->setFreqTarget ( (GSSynthOsc) voiceIdx , 0.0f , relRate );
		gsPtr->getSynth()->setAmplitude  ( (GSSynthOsc) voiceIdx , 0.0f );
	}
	else
	{
		gsPtr->getSynth()->setAmplitude  ( (GSSynthOsc) voiceIdx , amplitude );
	}
}
