#ifndef _GINSING_POLY_DEF
#define _GINSING_POLY_DEF

// ---------------------------------------------------------------------------------------------------------------------------------------------
// GinSingPoly - polyphonic instrument mode interface
// ---------------------------------------------------------------------------------------------------------------------------------------------
//
// Poly mode is a simplification of the synthesizer mode in that it sets up 6 identical oscillators (or voices) and mixes them equally to the
// output. The interface allows you to specify parameters per voice or globally (using voiceIdx = AllVoices ) for conveniences. Each voice has
// a selectable waveform, frequency or note, and an amplitude envelope. When begin() is called, the configuration creates these voices and sets
// them to a simple set of musical notes that are previewed via the preview() method; this serves as a good starting place for customizing the
// mode. In addition to basic tone generators, the voices can have simple modulation effects such as frequency distortion, pulse wave duty cycle,
// and portamento (pitch bending). The latter is a special case in that it uses a fixed volume rather than ADSR for amplitude.
//
// ---------------------------------------------------------------------------------------------------------------------------------------------
//
// As a simple example, this code will enter poly mode and trigger a simple musical chord:
//
//  GinSingPoly *poly = GS.getPoly();																	// get the poly mode class
//
//    poly->begin();																					// switch to poly mode operation
//
//    poly->setWaveform ( AllVoices , SINE );															// set all voices to use a sinusoidal waveform
//    poly->setEnvelope ( AllVoices , AT_100MS , 0.3f , DR_300MS , 0.3f , DR_2400MS , 0.0f );			// 30% volume 100ms rise, 2.4s fall
//    poly->setNote     ( 0 , A_3  );																	// set voice 0 to A 3rd octave
//    poly->setNote     ( 1 , CS_4 );																	// set voice 1 to C# 4th octave
//    poly->setNote     ( 2 , E_4  );																	// set voice 2 to E 4th octave
//
//    poly->trigger ( 0 ); poly->trigger ( 1 ); poly->trigger ( 2 );									// trigger the three notes ( ADSR envelope )
//    delay ( 3000 );																					// wait to hear them
//    poly->release ( 0 ); poly->release ( 1 ); poly->release ( 2 );									// trigger envelope rlease
//    delay ( 3000 );																					// wait for envelope release
//
// ---------------------------------------------------------------------------------------------------------------------------------------------



class GinSingPoly
{
		
public:

	
	// polyphonic voice slots
	
	enum PolyParams
	{
		NumVoices = 6   ,															// number of avialable simultaneous voices
		AllVoices = 0xff ,															// special voice index that applies to all voices
	};

	
	// polyphonic mode methods
	
	void begin         ();															// set up for polyphonic instrument mode
	void preview       ();															// preview default polyphonic mode setup
	
	void trigger       ( ubyte voiceIdx );											// trigger the ampiltude envelope for a voice
	void release       ( ubyte voiceIdx );											// release the amplitude envelope for a voice
	
	void setNote       ( ubyte voiceIdx , GSNote     noteIdx    );					// set the musical note for an oscillator
	void setWaveform   ( ubyte voiceIdx , GSWaveType waveform   );					// set the waveform type
	void setFreqDist   ( ubyte voiceIdx , float      distLevel  );					// set the frequency distortion level ( 0.0 to 1.0 )
	void setDutyCycle  ( ubyte voiceIdx , float      dutyCycle  );					// set the duty cycle for square wave ( 0.0 to 1.0 , 0.5 = square )
	
	void setPortamento ( ubyte voiceIdx , float amplitude , float relRate );		// set notes to bend from one to another at specified relative rate ( 0.0-1.0)
																					// envelope is not available when portamento is used
	
	void setTremolo    ( ubyte voiceIdx , float rateHz );							// set notes to amplitude modulate at the given frequency (Hz)
																					// envelope is not available when tremolo is used

	void setEnvelope   ( ubyte voiceIdx ,
					     GSAttackDur attackDur  , float attackAmp  , 				// set the ADSR envelope
					     GSDecRelDur decayDur   , float decayAmp   , 				// amplitudes vary from 0.0 to 1.0
					     GSDecRelDur releaseDur , float releaseAmp );			    // durations are limited to selections in GinSingDefs.h

};


#endif // _GINSING_POLY_DEF

