#ifndef _GINSINGSERIAL_H_DEF
#define _GINSINGSERIAL_H_DEF

// ---------------------------------------------------------------------------------------------------------------------------------------------
// GinSingSerial - serial interface to the SingSing Arudino Board
// ---------------------------------------------------------------------------------------------------------------------------------------------
// 
// This code has be modified from its original content to better fit into this software library.
// Our many thanks an gratitude to the original writers of this code. This library is being redistributed under
// the GNU Lesser Genral Public License as published by the Free Software Foundation version 2.1 or later.
// You should have received a copy of the GNU Lesser General Public License along with this library; if not, write 
// to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
// This code is based on a downloaded version of NewSoftSerial.h from http://arduiniana.org. We have decided to 
// encapsulate this version to avoid having to link in another library for convenience.
// We hope you actively support development efforts by these fine people.
//
// --------------------------------------------------------------------------------------------------------------------------------------

#include <inttypes.h>											// native variable types
#include <Stream.h>												// streaming control
#include "GinSingSerialMacros.h"								// useful macros for this lubrary


#define _SS_MAX_RX_BUFF  64										// RX buffer size ( in bytes )
#define _SS_VERSION      01										// software version of this library


// --------------------------------------------------------------------------------------------------------------------------------------
// GinSingSerial - library class that supports serial interface on any pin set
// --------------------------------------------------------------------------------------------------------------------------------------

class GinSingSerial : public Stream
{
	
public:
	
	GinSingSerial ();											// constructor
	~GinSingSerial();											// destructor	

	void         config ( uint8_t receivePin  ,					// configure the hardware connections for
	                      uint8_t transmitPin ,					// communication
	                      uint8_t ovfPin      );	
	
	bool         begin ( long speed );							// activate serial connection at the specified speed
	                     				
	bool         listen();										// listen for communication from the other side of the link
	void         end();											// terminate the serial connection
	
	bool         is_listening();								// true if data is availabe on the other side of the connection
	bool         rcvOverflow();									// true if the recieve buffer has been overrun
	bool         sndOverflow();									// true if the device has ben overrun by sends
	
	static int   library_version();								// get the current library version
	static void  enable_timer0 ( bool enable );					// enables hardware time for interrupt processing
	int          peek();										// check the contents of the receive buffer
	
	
#if defined(ARDUINO) && ARDUINO >= 100	
	virtual size_t write ( uint8_t byte );						// write a byte out to the serial connection
#else
	virtual void write ( uint8_t byte );						// write a byte out to the serial connection
#endif
	
	virtual int  read ();										// read the concents of the recieve buffer
	virtual int  available ();									// check to see if dadta is available
	virtual void flush ();										// flush the contents of the buffers
	 
	bool         readByte( uint8_t * dataByte );				// read single byte and return true if data is available

	
	static inline void handle_interrupt ();						// internal method to access interrupt handlers
	
	
private:
	
	// internal variables
	
    uint8_t           _receivePin;								// pin assigned to recieve data
    uint8_t           _receiveBitMask;							// bitmask applied to  recieved data
    volatile uint8_t *_receivePortRegister;						// chip register port for recieve
	
    uint8_t           _transmitBitMask;							// bitmask applied to send data
    volatile uint8_t *_transmitPortRegister;					// chip register port for transit

    uint16_t          _rx_delay_centering;						// temporal receive timing parameter
    uint16_t          _rx_delay_intrabit;						// temporal receive timing parameter
    uint16_t          _rx_delay_stopbit;						// temporal receive timing parameter
    uint16_t          _tx_delay;								// computed delay value for read
	
	uint8_t           _ovfPin;									// pin assigned for overflow control
    uint16_t          _rcv_overflow:1;							// true if received overload has occurred
	bool              _snd_overflow;							// true if send overflow has occurred
    uint16_t          _inverse_logic:1;							// true if using inverse log

	
  // static data
	
    static char             _receive_buffer[_SS_MAX_RX_BUFF];	// static recieve data ring buffer
    static volatile uint8_t _receive_buffer_tail;				// pointer to the tail of ring buffer
    static volatile uint8_t _receive_buffer_head;				// pointer tothe head of the ring buffer
	
    static GinSingSerial *active_object;						// internal pointer to the class oject

	
  // private methods
	
    void    recv();												// perform blocking read from the connection
    uint8_t rx_pin_read();										// read raw data from the RX pin
    void    tx_pin_write(uint8_t pin_state);					// send raw data to the TX pin
    void    setTX ( uint8_t transmitPin );						// assign transmit hardware pin
    void    setRX ( uint8_t receivePin );						// assign receive hardware pin
	void    setOVF ( uint8_t ovrPin );							// assign send overflow pin
		
    static inline void tunedDelay ( uint16_t delay );			// perform delay based on connection parameters


};

#endif // _GINSINGSERIAL_H_DEF
