// ---------------------------------------------------------------------------------------------------------------------------------------------
// GinSingSynth - wave synthesizer support for the Arduino GinSing shield
// ---------------------------------------------------------------------------------------------------------------------------------------------


#include "GinSing.h"


static ubyte _curBankIdx = 0;


// this macro allows the inteface to accept direct voice index (0-5) as well as bank/oscillator specification
// the former is used for poly mode, the latter for synth mode

#define OscIdxToVoiceIdx(OSC) ( ( OSC < OSC_1 || OSC == OSC_ALL ) ? OSC : ( ( OSC & 0x0f ) + 3 * _curBankIdx ) )
#define OscMaxCount 6


// commands

static GSCommand  _triggerCmds[] = { TriggerOsc_A1  , TriggerOsc_A2  , TriggerOsc_A3  , TriggerOsc_B1  , TriggerOsc_B2  , TriggerOsc_B3  };
static GSCommand  _releaseCmds[] = { ReleaseOsc_A1  , ReleaseOsc_A2  , ReleaseOsc_A3  , ReleaseOsc_B1  , ReleaseOsc_B2  , ReleaseOsc_B3  };
static GSCommand  _noteCmds   [] = { LoadNoteOsc_A1 , LoadNoteOsc_A2 , LoadNoteOsc_A3 , LoadNoteOsc_B1 , LoadNoteOsc_B2 , LoadNoteOsc_B3 };


// oscillator registers

static GSRegister _ctrlRegs[] = { A1_Control           , A2_Control           , A3_Control           , B1_Control           , B2_Control           , B3_Control     };
static GSRegister _frqRegs [] = { A1_FrequencyFine     , A2_FrequencyFine     , A3_FrequencyFine     , B1_FrequencyFine     , B2_FrequencyFine     , B3_FrequencyFine  };
static GSRegister _ampRegs [] = { A1_Amplitude         , A2_Amplitude         , A3_Amplitude         , B1_Amplitude         , B2_Amplitude         , B3_Amplitude   };
static GSRegister _atkRegs [] = { A1_EnvelopeAttack    , A2_EnvelopeAttack    , A3_EnvelopeAttack    , B1_EnvelopeAttack    , B2_EnvelopeAttack    , B3_EnvelopeAttack };
static GSRegister _dcyRegs [] = { A1_EnvelopeDecay     , A2_EnvelopeDecay     , A3_EnvelopeDecay     , B1_EnvelopeDecay     , B2_EnvelopeDecay     , B3_EnvelopeDecay };
static GSRegister _rlsRegs [] = { A1_EnvelopeRelease   , A2_EnvelopeRelease   , A3_EnvelopeRelease   , B1_EnvelopeRelease   , B2_EnvelopeRelease   , B3_EnvelopeRelease };
static GSRegister _envRegs [] = { A1_EnvelopeControl   , A2_EnvelopeControl   , A3_EnvelopeControl   , B1_EnvelopeControl   , B2_EnvelopeControl   , B3_EnvelopeControl };
static GSRegister _frtRegs [] = { A1_FrequencyTargetLow, A2_FrequencyTargetLow, A3_FrequencyTargetLow, B1_FrequencyTargetLow, B2_FrequencyTargetLow, B3_FrequencyTargetLow };
static GSRegister _frxRegs [] = { A1_FrequencyXLow     , A2_FrequencyXLow     , A3_FrequencyXLow     , B1_FrequencyXLow     , B2_FrequencyXLow     , B3_FrequencyXLow };
static GSRegister _amptRegs[] = { A1_AmplitudeTarget   , A2_AmplitudeTarget   , A3_AmplitudeTarget   , B1_AmplitudeTarget   , B2_AmplitudeTarget   , B3_AmplitudeTarget };
static GSRegister _atxRegs [] = { A1_AmplitudeXLow     , A2_AmplitudeXLow     , A3_AmplitudeXLow     , B1_AmplitudeXLow     , B2_AmplitudeXLow     , B3_AmplitudeXLow };


// mixer registers

static GSRegister _mixCtrl0Regs [] = { A_MixControl_0 , B_MixControl_0 };
static GSRegister _mixCtrl1Regs [] = { A_MixControl_1 , B_MixControl_1 };
static GSRegister _distRegs     [] = { A_OscDistort_1 , A_OscDistort_2 , A_OscDistort_3 , B_OscDistort_1 , B_OscDistort_2 , B_OscDistort_3 };
static GSRegister _pwmRegs      [] = { A_OscPWM_1     , A_OscPWM_2     , A_OscPWM_3     , B_OscPWM_1     , B_OscPWM_2     , B_OscPWM_3     };


// oscillator control bits

#define OC_WaveTypeCtrlBits     0x07									// [0-2|4] wave type | overflow
#define OC_WaveModeCtrlBits     0x08									// [3    ] waveform absolute value or symmetrical
#define OC_ResetBits            0x10									// [4    ] reset the oscillator (requires full byte)
#define OC_BendCtrlBits         0x20									// [5    ] enable or disable frequency bend
#define OC_UnusedBits           0x40									// [6    ] no function
#define OC_EnableBits           0x80									// [7    ] enable or disable oscillator


// oscillator envelope control bits

#define EC_StatusBits          0x03										// [0-1] read only - envelope phase
#define EC_FrqTgtCtrlBits      0x04										// [2  ] frq tgt bit | bit 3 frq ramp bit   ( need ramp bit as well )
#define EC_AmpTgtCtrlBits      0x08										// [3  ] amp tgt bit | bit 7 amp ramp bit   ( need ramp bit as well )
#define EC_FrqRampCtlBits      0x10										// [4  ] enable or disable frequency ramp
#define EC_AmpRampCtrlBits     0x20										// [5  ] enable or disable amplitude ramp
#define EC_RunningBits         0x40										// [6  ] read only - envelope running
#define EC_ActiveBits          0x80										// [7  ] read only - envelpe active


// mixer 0 control bits ( these are exposed as part of patch byte 0 )

#define M0_Osc1ToMixer		    OSC_1_TO_MIXER							// [0  ] send oscillator 1 output to the main mixer
#define M0_Osc2ToMixer          OSC_2_TO_MIXER							// [1  ] send oscillator 2 output to the main mixer
#define M0_Osc3ToMixer          OSC_3_TO_MIXER							// [2  ] send oscillator 3 output to the main mixer
#define M0_OscB1AmpModMixer     OSC_B1_AMPMOD_MIXER						// [3  ] use bank B oscillator 1 output to amplitude modulate main mixer
#define M0_OscB1AmpModMixerHalf OSC_B1_AMPMOD_MIXER_50PERC				// [4  ] modified to OSC_B1_AMPMOD_MIXER to attenuate signal by 50%
#define M0_OscB1SyncAllOscs     OSC_B1_SYNC_A1_A2_A3					// [5  ] use bank B oscillator 1 to sync all bank A oscillators
#define M0_Unused               0xc0									// [6-7] no function


// mixer 1 control bits ( these are exposed as part of patch byte 1 )

#define M1_OscB1PWMOscA1		 ( OSC_B1_PWM_OSC_A1 >> 8 )				// [0 ] use bank B oscillator 1 to pulse modulate oscillator 1 (either bank)
#define M1_OscB2PWMOscA2		 ( OSC_B2_PWM_OSC_A2 >> 8 )				// [1 ] use bank B oscillator 2 to pulse modulate oscilaltor 2 (either bank)
#define M1_OscB3PWMOscA3		 ( OSC_B3_PWM_OSC_A3 >> 8 )				// [2 ] use bank B oscillator 3 to pulse modulate oscilaltor 3 (either bank)#define M1_Osc2AmpModOsc1       OSC_2_AMPMOD_OSC_1						// use oscillator 2 to amplitude modulate oscillator 1
#define M1_Osc2AmpModOsc1		 ( OSC_2_AMPMOD_OSC_1 >> 8 )			// [3 ] use oscillator 2 to amplitude modulate oscillator 1
#define M1_Osc2AmpModOsc1Half    ( OSC_2_AMPMOD_OSC_1_50PERC >> 8 )		// [4 ] use oscillator 2 to amplitude modulate oscillator 1 at 50%
#define M1_Osc2SyncOsc1          ( OSC_2_SYNC_OSC_1	>> 8 )				// [5 ] use oscillator 2 to sync oscillator 1
#define M1_Osc3FrqModOsc1        ( OSC_3_FRQMOD_OSC_1 >> 8 )			// [6 ] use oscillator 3 to frequency modulate oscillator 1
#define M1_Osc3FrqModOsc1Half    ( OSC_3_FRQMOD_OSC_1_50PERC >> 8 )		// [7 ] use oscillator 3 to frequency modulate oscillator 1 at 50%


// helper functions ( set regs and send commands to all oscillators )

void _setAllOscCmd0 ( GSCommand *cmds );
void _setAllOscCmd1 ( GSCommand *cmds , ubyte arg1 );
void _setAllOscCmd3 ( GSCommand *cmds , ubyte arg1 , ubyte arg2 , ubyte arg3 );

void _setAllOscReg1Byte     ( GSRegister *regs , ubyte value );
void _setAllOscReg2Bytes    ( GSRegister *regs , ubyte val1, ubyte val2 );
void _setAllOscReg3Bytes    ( GSRegister *regs , ubyte val1, ubyte val2, ubyte val3 );
void _setAllOscReg1ByteMask ( GSRegister *regs , ubyte value , ubyte mask );


// ---------------------------------------------------------------------------------------------------------------------------------------------
// begin - configure the system for polyphonic instrument mode
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::begin()
{
	
	// reset the system
	
	gsPtr->getMaster()->reset ();
	
	
	// set up the default properties for the oscillators
	
	setEnvelope   ( OSC_ALL , AT_2MS , 0.60f , DR_2MS , 0.60f , DR_575MS , 0.0f  );			        // 60% volume 2ms rise, 0.5s fall
    enable        ( OSC_ALL , true );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// preview - preview poly mode
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::preview()
{
	setEnvelope   ( OSC_ALL , AT_1000MS , 0.25f , DR_2MS , 0.25f , DR_1500MS , 0.0f  );
	
	selectBank   ( BANK_A );
	setPatch     ( OSC_B2_PWM_OSC_2 | OSC_2_TO_MIXER );
	
    setWaveform  ( OSC_2 , PULSE );
	setFrequency ( OSC_2 , 40.0f );
    trigger      ( OSC_2  );
	
	
    selectBank   ( BANK_B );
    setPatch     ( OSC_B2_PWM_OSC_2 | OSC_1_TO_MIXER );
	
    setWaveform  ( OSC_2 , TRIANGLE );
    setWavemode  ( OSC_2 , POSITIVE );
    setFrequency ( OSC_2 , 0.5f );
    trigger      ( OSC_2  );

		
	// wait a second to hear it
	
	delay ( 3000 );
	
	
	// release and wait for envelope to finish
	
	selectBank ( BANK_A ); release ( OSC_2 );
	selectBank ( BANK_B ); release ( OSC_1 ); release ( OSC_2 );

	delay ( 3000 );
	
	
	// restore to factory settings
	
	begin();
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// enable - enable or disable an oscillator
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::enable ( GSSynthOsc oscIdx , bool enable )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	if ( oscIdx != OSC_ALL ) 
	{
		if ( enable )  gsPtr->sendCommand ( WriteOneByteWithMask , _ctrlRegs [ voiceIdx ] , OC_EnableBits , OC_EnableBits );
		else           gsPtr->sendCommand ( WriteOneByteWithMask , _ctrlRegs [ voiceIdx ] , 0x0           , OC_EnableBits );
	}
	else
	{
		if ( enable ) _setAllOscReg1ByteMask ( _ctrlRegs , OC_EnableBits , OC_EnableBits );
		else          _setAllOscReg1ByteMask ( _ctrlRegs , 0x0           , OC_EnableBits );
	}
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// trigger - trigger the ampltiude envelope for a voice
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::trigger ( GSSynthOsc oscIdx )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// set the enable bit(s)
	
	if ( oscIdx != OSC_ALL ) gsPtr->sendCommand ( _triggerCmds [ voiceIdx ] );
    else                     _setAllOscCmd0     ( _triggerCmds );
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// release - release the ampltiude envelope
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::release ( GSSynthOsc oscIdx )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// send the release command(s)
	
	if ( oscIdx != OSC_ALL ) gsPtr->sendCommand ( _releaseCmds [ voiceIdx ] );
    else                     _setAllOscCmd0     ( _releaseCmds );
	
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// selectBank - select the target bank for other functions
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::selectBank ( GSSynthBank bankIdx )
{
	// set the current bank used by other methods
	
	_curBankIdx = bankIdx;
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setPatch - configure the curernt bank patch
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setPatch ( uint patchConfig )
{
	// separate the bits into each mixer control value
	
	ubyte mixCtrl0 = ( patchConfig      ) & 0xff;
	ubyte mixCtrl1 = ( patchConfig >> 8 ) & 0xff;
	
	// send the mixer control registers
	
	gsPtr->sendCommand ( WriteOneByte , _mixCtrl0Regs [ _curBankIdx ] , mixCtrl0 );
	gsPtr->sendCommand ( WriteOneByte , _mixCtrl1Regs [ _curBankIdx ] , mixCtrl1 );
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setNote - set the musical note for a voice
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setNote ( GSSynthOsc oscIdx , GSNote noteIdx )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);

	// send set note command(s)
	
	if ( voiceIdx != OSC_ALL )   gsPtr->sendCommand ( _noteCmds [ voiceIdx ] , noteIdx );
	else                         _setAllOscCmd1     ( _noteCmds              , noteIdx );
	
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setFrequency - set the ptich (Hz) for a voice
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setFrequency ( GSSynthOsc oscIdx , float freqHz )
{

	// convert frequency to 24 bit value
	
	ulong freqInt = (ulong) ( freqHz / 0.0009313226 );
	
	
	// set frequency with full resolution
	
	setFreqVal ( oscIdx , freqInt );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setFreqVal - set the ptich for a voice at maximum resolution
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setFreqVal ( GSSynthOsc oscIdx , ulong freqVal )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	ubyte freqFine = (unsigned byte) ( ( freqVal & 0x000000ff )       );
    ubyte freqLo   = (unsigned byte) ( ( freqVal & 0x0000ff00 ) >>  8 );
    ubyte freqHi   = (unsigned byte) ( ( freqVal & 0x00ff0000 ) >> 16 );
	
	// set the frequency register(s)
	
	if ( voiceIdx != OSC_ALL ) gsPtr->sendCommand  ( WriteThreeBytes , _frqRegs [ voiceIdx ] , freqFine , freqLo , freqHi );
	else                       _setAllOscReg3Bytes (                   _frqRegs              , freqFine , freqLo , freqHi );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmplitude - set the mixer output ampltiude
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setAmplitude ( GSSynthOsc oscIdx , float amplitude )
{
	// compute amplitude as a signed byte
	
	sbyte ampByte = (sbyte) ( 0x7f * amplitude );
	setAmplitudeVal ( oscIdx , ampByte );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmplitudeVal - set the mixer output amplitude by integer
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setAmplitudeVal ( GSSynthOsc oscIdx , sbyte ampVal )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);

	// set the amplitude register(s)
	
	if ( voiceIdx != OSC_ALL ) gsPtr->sendCommand ( WriteOneByte , _ampRegs [ voiceIdx ] , ampVal );
	else                       _setAllOscReg1Byte (                _ampRegs              , ampVal );
	
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setWaveform - set the output waveform
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setWaveform  ( GSSynthOsc oscIdx , GSWaveType waveform )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// set the oscillator control bits ( low 3 bits )
	
	ubyte waveByte;
	
	switch ( waveform )
	{
		case SINE     : waveByte = 0x00; break;
		case TRIANGLE : waveByte = 0x01; break;
		case SAWTOOTH : waveByte = 0x02; break;
		case RAMP     : waveByte = 0x03; break;
		case PULSE    : waveByte = 0x04; break;
		case NOISE    : waveByte = 0x05; break;
		case LEVEL    : waveByte = 0x00; break;
	}
	

	// write the waveform type bits (0-3)
		
	if ( voiceIdx != OSC_ALL )  gsPtr->sendCommand      ( WriteOneByteWithMask , _ctrlRegs [ voiceIdx ] , waveByte , OC_WaveTypeCtrlBits );
	else                        _setAllOscReg1ByteMask  (                        _ctrlRegs              , waveByte , OC_WaveTypeCtrlBits );
						 
						 
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setWavemode - set the output waveform mode
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setWavemode  ( GSSynthOsc oscIdx , GSWaveMode wavemode  )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);

	
	// set the wave mode control bit(s)
	
	if ( voiceIdx != OSC_ALL )
	{
		if ( wavemode == SYMMETRIC ) gsPtr->sendCommand ( WriteOneByteWithMask , _ctrlRegs [ voiceIdx ] , 0x0              , OC_WaveModeCtrlBits );
		else                         gsPtr->sendCommand ( WriteOneByteWithMask , _ctrlRegs [ voiceIdx ] , OC_WaveModeCtrlBits , OC_WaveModeCtrlBits );
	}
	else
	{
		if ( wavemode == SYMMETRIC ) _setAllOscReg1ByteMask ( _ctrlRegs , 0x0              , OC_WaveModeCtrlBits );
		else                         _setAllOscReg1ByteMask ( _ctrlRegs , OC_WaveModeCtrlBits , OC_WaveModeCtrlBits );
	}
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// enableOverflow - enable / disable the overflow on waveform cycling
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::enableOverflow ( GSSynthOsc oscIdx , bool enable )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// set the overflow/reset control bit
	
	if ( voiceIdx != OSC_ALL )
	{
		if ( enable )  gsPtr->sendCommand ( WriteOneByteWithMask , _ctrlRegs [ voiceIdx ]  , OC_ResetBits , OC_ResetBits  );
		else           gsPtr->sendCommand ( WriteOneByteWithMask , _ctrlRegs [ voiceIdx ]  , 0x0          , OC_ResetBits  );
	}
	else {
		
		if ( enable ) _setAllOscReg1ByteMask ( _ctrlRegs , OC_ResetBits , OC_ResetBits );
		else          _setAllOscReg1ByteMask ( _ctrlRegs , 0x0          , OC_ResetBits );
	}
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setFreqDist - set the waveform frequency distortion
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setFreqDist ( GSSynthOsc oscIdx , float distLevel )
{
	// convert from float to unsigned byte
	
	ubyte distByte = (ubyte) ( 255.0f * distLevel );
	
	setFreqDistVal ( oscIdx , distByte );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setFreqDistVal - set the waveform frequency distortion by unsigned byte
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setFreqDistVal ( GSSynthOsc oscIdx , ubyte distVal )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// set the distortion register(s)
	
	if ( voiceIdx != OSC_ALL )  gsPtr->sendCommand ( WriteOneByte , _distRegs [ voiceIdx ] , distVal );
	else                        _setAllOscReg1Byte (                _distRegs              , distVal );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setDutyCycle - set the waveform duty cycle ( square wave only )
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setDutyCycle ( GSSynthOsc oscIdx , float dutyCycle )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);

	
	// convert from float to unsigned byte
	
	sbyte pwmByte = (ubyte) ( 0x7f * dutyCycle );	
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setDutyCycleVal - set the waveform duty cycle as signed byte
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setDutyCycleVal ( GSSynthOsc oscIdx , sbyte dcVal )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
		
	// set the PWM register
	
	if ( voiceIdx != OSC_ALL ) gsPtr->sendCommand ( WriteOneByte , _pwmRegs [ voiceIdx ] , dcVal );
	else                       _setAllOscReg1Byte (                _pwmRegs              , dcVal );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setEnvelope - set the ADSR envelope parameters
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setEnvelope  ( GSSynthOsc  oscIdx ,
								  GSAttackDur attackDur  , float attackAmp  ,
				                  GSDecRelDur decayDur   , float decayAmp   ,
				                  GSDecRelDur releaseDur , float releaseAmp )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);

	
	// convert amplitude to unsigned bytes
	
	ubyte atkByte = ( (ubyte) ( 0x0f * attackAmp   ) );
	ubyte dcyByte = ( (ubyte) ( 0x0f * decayAmp    ) );
	ubyte rlsByte = ( (ubyte) ( 0x0f * releaseAmp  ) );
	
	setEnvelopeVal ( oscIdx , attackDur   , atkByte ,
					          decayDur    , dcyByte ,
					          releaseDur  , rlsByte );

}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setEnvelope - set the ADSR envelope parameters using unsigned bytes
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::setEnvelopeVal  ( GSSynthOsc  oscIdx ,
								     GSAttackDur attackDur  , ubyte attackAmpVal  ,
								     GSDecRelDur decayDur   , ubyte decayAmpVal   ,
								     GSDecRelDur releaseDur , ubyte releaseAmpVal )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	
	// construct ADR bytes ( hi 4 bits amplitude, low 4 bits duration )
	
	ubyte atkByte = ( ( attackAmpVal   ) << 4 ) + attackDur;
	ubyte dcyByte = ( ( decayAmpVal    ) << 4 ) + decayDur;
	ubyte rlsByte = ( ( releaseAmpVal  ) << 4 ) + releaseDur;
	
	
	// set the ADSR register(s)
	
	if ( voiceIdx != OSC_ALL )
	{
		gsPtr->sendCommand ( WriteOneByte , _atkRegs [ voiceIdx ] , atkByte );
		gsPtr->sendCommand ( WriteOneByte , _dcyRegs [ voiceIdx ] , dcyByte );
		gsPtr->sendCommand ( WriteOneByte , _rlsRegs [ voiceIdx ] , rlsByte );
	}
	else
	{
		_setAllOscReg1Byte ( _atkRegs , atkByte );
		_setAllOscReg1Byte ( _dcyRegs , dcyByte );
		_setAllOscReg1Byte ( _rlsRegs , rlsByte );
	}
	
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// enableFrequencyTarget - enable or disable frequency target
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingSynth::enableFreqTarget ( GSSynthOsc oscIdx , bool enable )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);

	// set frequency target bits
	
	if ( enable ) _setAllOscReg1ByteMask ( _envRegs , EC_FrqTgtCtrlBits , EC_FrqTgtCtrlBits );
	else          _setAllOscReg1ByteMask ( _envRegs , 0x00              , EC_FrqTgtCtrlBits );

}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setFrequencyTarget - set a frequency target and the rate at which it will track towards it
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setFreqTarget  ( GSSynthOsc oscIdx , float freqHz , float relRate )
{

	// convert target frequency to 2 byte value
	// scalar is computed based on maximum frequency into
	// signed 2 byte ( 7812.499 / 2^15 ) = 0.238418548
	
	uint freqInt = (uint) ( freqHz / 0.238418548 );
	uint xrInt   = (uint) ( 0x7fff * relRate );
	
	setFreqTargetVal ( oscIdx , freqInt , xrInt );
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setFrequencyTargetVal - set frequency target as an integer
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setFreqTargetVal  ( GSSynthOsc oscIdx , uint freqVal , uint rateVal )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// convert target frequency to 2 byte value
	
	ubyte freqLo   = (ubyte)(   freqVal & 0x00ff );
	ubyte freqHi   = (ubyte)( ( freqVal & 0xff00 ) >> 8 );	
	
	
	// convert target rate to 2 byte value
	
	ubyte xrLo     = (ubyte)(   rateVal & 0x00ff );
	ubyte xrHi     = (ubyte)( ( rateVal & 0x7f00 ) >> 8 );
	
	// send the target, rate, and oscilator frequency target bits (2|4)
	
	if ( voiceIdx != OSC_ALL )
	{
		gsPtr->sendCommand ( WriteTwoBytes , _frtRegs [ voiceIdx ] , freqLo , freqHi );
		gsPtr->sendCommand ( WriteTwoBytes , _frxRegs [ voiceIdx ] , xrLo   , xrHi   );
	}
	else {
		_setAllOscReg2Bytes    ( _frtRegs , freqLo , freqHi );
		_setAllOscReg2Bytes    ( _frxRegs , xrLo   , xrHi   );
	}
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// enableFreqRamp - enable or disable frequency ramping
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::enableFreqRamp  ( GSSynthOsc oscIdx , bool enable )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// set frequency ramp bit
	
	if ( voiceIdx != OSC_ALL ) {
	
		if ( enable )
			gsPtr->sendCommand ( WriteOneByteWithMask , _envRegs [ voiceIdx ] , EC_FrqRampCtlBits , EC_FrqRampCtlBits );
		else
			gsPtr->sendCommand ( WriteOneByteWithMask , _envRegs [ voiceIdx ] , 0x0               , EC_FrqRampCtlBits );
	}
	else {
		
		if ( enable ) _setAllOscReg1ByteMask ( _envRegs , EC_FrqRampCtlBits , EC_FrqRampCtlBits );
		else          _setAllOscReg1ByteMask ( _envRegs , 0x0               , EC_FrqRampCtlBits );
	}
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setFreqRamp - set a frequency ramp sweep at a given rate
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setFreqRamp  ( GSSynthOsc oscIdx , float relRate )
{
	// convert target rate to a 2 byte integer
	
	uint xrInt = (uint) ( 0x7fff * relRate );
	setFreqRampVal ( oscIdx , xrInt );
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setFreqRampVal - set a frequency ramp sweep as a
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setFreqRampVal  ( GSSynthOsc oscIdx , uint relRateVal )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// convert target rate to 2 byte values
	
	ubyte xrLo = (ubyte)(   relRateVal & 0x00ff );
	ubyte xrHi = (ubyte)( ( relRateVal & 0x7f00 ) >> 8 );

	
	// set the transition rate and frequency ramp bit (4)
		
	if ( voiceIdx != OSC_ALL )
	{
		gsPtr->sendCommand ( WriteTwoBytes , _frxRegs [ voiceIdx ] , xrLo , xrHi );
	}
	else
	{
		_setAllOscReg2Bytes ( _frxRegs , xrLo , xrHi );
	}
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// enableAmpTarget - enable or disable amplitude targeting
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::enableAmpTarget  ( GSSynthOsc oscIdx , bool enable )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// set the amp target bit
	
	if ( voiceIdx != OSC_ALL ) 
	{
		if ( enable )
			gsPtr->sendCommand ( WriteOneByteWithMask , _envRegs [ voiceIdx ] , EC_AmpTgtCtrlBits , EC_AmpTgtCtrlBits );
		else
			gsPtr->sendCommand ( WriteOneByteWithMask , _envRegs [ voiceIdx ] , 0x0               , EC_AmpTgtCtrlBits );
		
	}
	else
	{
		if ( enable ) _setAllOscReg1ByteMask ( _envRegs  , EC_AmpTgtCtrlBits , EC_AmpTgtCtrlBits );
		else          _setAllOscReg1ByteMask ( _envRegs  , 0x0               , EC_AmpTgtCtrlBits );
	}
	
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmpTarget - set an amplitude target and the rate at which it will track towards it
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setAmpTarget  ( GSSynthOsc oscIdx , float ampTgt , float relRate )
{
	// convert floats to ints
	
	sbyte ampByte = (sbyte) ( 0x7f * ampTgt );
	uint xrInt   = (uint) ( 0x7fff * relRate );
	
	setAmpTargetVal ( oscIdx , ampByte , xrInt );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmpTargetVal - set an amplitude target as an integer
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setAmpTargetVal  ( GSSynthOsc oscIdx , sbyte ampVal , uint relRateVal )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// convert rate to 2 byte value
	
	ubyte xrLo = (ubyte) ( ( relRateVal & 0x00ff ) );
	ubyte xrHi = (ubyte) ( ( relRateVal & 0x7f00 ) >> 8 );

	
	// set the target and rate bits
	
	if ( voiceIdx != OSC_ALL ) 
	{
		gsPtr->sendCommand ( WriteOneByte  , _amptRegs[ voiceIdx ] , ampVal );
		gsPtr->sendCommand ( WriteTwoBytes , _atxRegs [ voiceIdx ] , xrLo , xrHi );
	}
	else
	{
		_setAllOscReg1Byte     ( _amptRegs , ampVal      );
		_setAllOscReg2Bytes    ( _atxRegs  , xrLo , xrHi );
	}
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// enableAmpRamp - enable or disable amplitude ramping
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::enableAmpRamp  ( GSSynthOsc oscIdx , bool enable )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
		
	
	// set the amp ramp bit
		
	if ( voiceIdx != OSC_ALL ) 
	{
		if ( enable ) 
			gsPtr->sendCommand ( WriteOneByteWithMask , _envRegs [ voiceIdx ] , EC_AmpRampCtrlBits , EC_AmpRampCtrlBits );
		else 
			gsPtr->sendCommand ( WriteOneByteWithMask , _envRegs [ voiceIdx ] , 0x0                , EC_AmpRampCtrlBits );
	}
	else
	{
		if ( enable ) 
			_setAllOscReg1ByteMask ( _envRegs , EC_AmpRampCtrlBits , EC_AmpRampCtrlBits );
		else
			_setAllOscReg1ByteMask ( _envRegs , 0x0                , EC_AmpRampCtrlBits );
			
	}
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmpRamp - set an ampltiude ramp sweep at a given rate
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setAmpRamp  ( GSSynthOsc oscIdx , float relRate )
{
	// convert rate to a 2 byte integer
	
	uint xrInt = (uint) ( 0x7fff * relRate );
	
	setAmpRampVal ( oscIdx , xrInt );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setAmpRampVal - set amplitude ramp parameters as integers
// ---------------------------------------------------------------------------------------------------------------------------------------------


void GinSingSynth::setAmpRampVal ( GSSynthOsc oscIdx , uint relRateVal )
{
	ubyte voiceIdx = OscIdxToVoiceIdx(oscIdx);
	
	// convert rate to 2 byte value
	
	ubyte xrLo = (ubyte) ( ( relRateVal & 0x00ff ) );
	ubyte xrHi = (ubyte) ( ( relRateVal & 0x7f00 ) >> 8 );
	
	
	// set the rate bits
	
	if ( voiceIdx != OSC_ALL ) 
	{
		gsPtr->sendCommand ( WriteTwoBytes , _atxRegs [ voiceIdx ] , xrLo , xrHi );
	}
	else
	{
		_setAllOscReg2Bytes    ( _atxRegs  , xrLo , xrHi );
	}
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// _SetAllOscReg1Byte - set all oscillator registers to the same 1 byte value
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllOscReg1Byte ( GSRegister *regs , ubyte value )
{	
	for ( int loopIdx = 0; loopIdx < OscMaxCount; loopIdx++ )
		gsPtr->sendCommand ( WriteOneByte , regs [ loopIdx ] , value );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// _setAllOscReg1ByteMask - send all oscillator registers with the same 1 byte masked value
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllOscReg1ByteMask ( GSRegister *regs , ubyte value , ubyte mask )
{
	for ( int loopIdx = 0; loopIdx < OscMaxCount; loopIdx++ )
		gsPtr->sendCommand ( WriteOneByteWithMask , regs [ loopIdx ] , value , mask );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// _SetAllOscReg2Bytes - set all oscillator registers to the same 2 byte value
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllOscReg2Bytes ( GSRegister *regs , ubyte val1, ubyte val2 )
{	
	for ( int loopIdx = 0; loopIdx < OscMaxCount; loopIdx++ )
		gsPtr->sendCommand ( WriteTwoBytes , regs [ loopIdx ] , val1, val2 );
}	


// ---------------------------------------------------------------------------------------------------------------------------------------------
// _SetAllOscReg3Bytes - set all oscillator registers to the same 3 byte value
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllOscReg3Bytes ( GSRegister *regs , ubyte val1, ubyte val2, ubyte val3 )
{	
	for ( int loopIdx = 0; loopIdx < OscMaxCount; loopIdx++ )
		gsPtr->sendCommand ( WriteThreeBytes , regs [ loopIdx ] , val1, val2, val3 );
}	


// ---------------------------------------------------------------------------------------------------------------------------------------------
// _setAllOscCmd0 - send all oscillator a non-argument command
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllOscCmd0 ( GSCommand *cmds )
{	
	for ( int loopIdx = 0; loopIdx < OscMaxCount; loopIdx++ ) 
		gsPtr->sendCommand ( cmds [ loopIdx ] );
}	

// ---------------------------------------------------------------------------------------------------------------------------------------------
// _setAllOscCmd1 - send all oscillator a non-argument command
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllOscCmd1 ( GSCommand *cmds , ubyte arg1 )
{
	for ( int loopIdx = 0; loopIdx < OscMaxCount; loopIdx++ )
		gsPtr->sendCommand ( cmds [ loopIdx ] , arg1 );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// _setAllOscCmd3 - send all oscillator a non-argument command
// ---------------------------------------------------------------------------------------------------------------------------------------------

void _setAllOscCmd3 ( GSCommand *cmds , ubyte arg1 , ubyte arg2 , ubyte arg3 )
{
	for ( int loopIdx = 0; loopIdx < OscMaxCount; loopIdx++ )
		gsPtr->sendCommand ( cmds [ loopIdx ] , arg1 , arg2 , arg3 );
}

