#ifndef _GINSING_SYNTH_DEF
#define _GINSING_SYNTH_DEF

// ---------------------------------------------------------------------------------------------------------------------------------------------
// GinSingSynth - waveform synthesizer mode interface
// ---------------------------------------------------------------------------------------------------------------------------------------------
//
// In synth mode, the 6 oscillators are configured into two functional banks that can be mixed independantly ( bank A, bank B )
// Each bank contains 3 oscillators each ( 1,2,3 ), and each has its own specific functionality as either a basic tone generator, 
// an amplitude modulator, and a frequency modulator; the latter two oscillators manipulate the waveform from the tone generator.
//
//
//
//                    BANK    (A/B)          
//      ------------------------------------------
//
//                         OSC 1 (TONE)  -> |  M           Although all oscillators can be sent to the main mix, oscillator 2 is primarily
//                             ^    ^       |  I           used to amplitude modulate oscillator 1, and oscillator 3 is primarily used to
//                             |    |       |  X           frequency modulate oscillator 1. This standard configuration can be represented as:
//       OSC 2 (AMPMOD) -------+ -- |-----> |  E
//       OSC 3 (FRQMOD) ------------+-----> |  R               [OSC] -> [MIX]      [OSC2] -> AMPMOD [OSC1]       [OSC3] -> FRQMOD [OSC1]
//
//
//  In addition to to this standard patching, there are two special patches that both utilize the bank B oscillators to modulate either
//  the mixers or bank A oscillators in special ways. Ther are as follows:
//
//  Bank B oscillators can be used to pulse width modulate the Bank A oscillators. There is a one-to-one assignment such that B1 modulates A1,
//  B2 modulates A2, and B3 modulates A3 respectively. Each of these modulations can be controlled independantly. This can represented as:
//
//   [B OSC] -> PWM [A OSC]
//
//  Bank B oscillator 1 has two special features that allow it to amplitude modulate the mixer in either bank and/or sync its waveform to all other
//  oscillators in either bank. This allows B1 to effect entire bank volume and/or to time align oscillator waveform starts. This can be
//  represented as :
//
//   [B1 OSC] -> AMPMOD [MIX]     [B1 OSC] -> SYNC [OSCS]
//
// ---------------------------------------------------------------------------------------------------------------------------------------------
//
// Here is a simple code example that sets up two tone oscillators - the first oscillator (#2) on bank A is sent to the mixer and is pulse width
// modulated via the waveform from oscillator (#2) on bank B:
//
// GinSinSynth *s = GS.getSynth();																// get the synth mode interface
// s->begin();																					// switch into synth mode
//
// s->setEnvelope  ( OSC_ALL , AT_800MS , 0.25f , DR_300MS , 0.25f , DR_8400MS , 0.0f  );		// set all envelopes (800ms->25%|300ms->25%|8.4s->0%)
// 
// s->selectBank   ( BANK_A );																	// set up bank A
// s->setPatch     ( OSC_B2_PWM_OSC_A2 | OSC_2_TO_MIXER );										// send output to mixer and modulate from B2
// s->setWaveform  ( OSC_2 , PULSE );															// use a pulse waveform
// s->setFrequency ( OSC_2 , 40.0f );															// set frequency to 40 Hz
// s->trigger      ( OSC_2  );																	// trigger oscillator
// 
// s->selectBank   ( BANK_B );																	// set up bank B
// s->setPatch     ( OSC_B2_PWM_OSC_A2 | OSC_1_TO_MIXER );										// send otuput to mixer and modulate A2
// s->setWaveform  ( OSC_2 , TRIANGLE );														// set to a triangle wave
// s->setWavemode  ( OSC_2 , POSITIVE );														// offet wave to only go positive to zero
// s->setFrequency ( OSC_2 , 0.5f );															// set period to 2 seconds
// s->trigger      ( OSC_2  );																	// trigger oscillator
// 
// delay ( 3000 );																				// wait to hear the sound
// s->selectBank ( BANK_A ); s->release ( OSC_2 );												// release bank A oscillators
// s->selectBank ( BANK_B ); s->release ( OSC_1 ); s->release ( OSC_2 );						// release bank B oscillators
// 
// delay ( 3000 );																				// wait for the envelope to relase
// 
//
//
// To obtain the maximum possible frequency resolution, you may specify the frequency as a 4 byte unsigned integer via the setFreqVal() method. 
// The conversion from integer to frequency is as follows:
//
// frequency (Hz) = freqInt * 0.0009313226 ,  so for example A 440.0 Hz = 440.0 / 0.009313226 = 427446
// frequency can range from 0.0009 Hz ( 1 ) to 7,182.448 Hz ( 7712094 )
//
// ---------------------------------------------------------------------------------------------------------------------------------------------


class GinSingSynth
{
		
public:

		
	// synth mode methods
	
	void begin         ();																// set up for wave synthesizer mode
	void preview       ();																// preview default wave synthesizer mode setup
	
	void selectBank    ( GSSynthBank  bankIdx );										// select the bank that the functions operate on
	void setPatch      ( uint         patchConfig );									// configure the software oscillator patches
	
	
	// oscillator functions
	
	void trigger         ( GSSynthOsc oscillator  );									// trigger the ampiltude envelope for an oscillator
	void release         ( GSSynthOsc oscillator  );									// release the amplitude envelope for an oscillator
	void enable          ( GSSynthOsc oscillator , bool       enable );					// enable or disable an oscillator

	void setWaveform     ( GSSynthOsc oscillator , GSWaveType waveform  );				// set the waveform type
	void setWavemode     ( GSSynthOsc oscillator , GSWaveMode wavemode  );				// set the waveform mode
	void setNote         ( GSSynthOsc oscillator , GSNote     noteIdx   );				// set the musical note for an oscillator
	void enableOverflow  ( GSSynthOsc oscillator , bool       enable );					// enable or disable wavetable overflow handling
	
	void setFrequency    ( GSSynthOsc oscillator , float      freqHz    );				// set the frequency (Hz) for an oscillator
	void setFreqVal      ( GSSynthOsc oscillator , ulong      freqVal   );				// set the frequency value ( see above )
	
	void setAmplitude    ( GSSynthOsc oscillator , float      amplitude );				// set wave amplitude ( -1.0 to 1.0, 0.0 = mute )
	void setAmplitudeVal ( GSSynthOsc oscillator , sbyte      ampVal );					// set wave amplitude as a signed byte
	
	void setFreqDist     ( GSSynthOsc oscillator , float      distLevel );				// set the frequency distortion level ( 0.0 to 1.0 )
	void setFreqDistVal  ( GSSynthOsc oscillator , ubyte      distVal );				// set the frequency distortion as an unsigned byte
	
	void setDutyCycle    ( GSSynthOsc oscillator , float      dutyCycle  );				// set the duty cycle for pulse wave ( -1.0 to 1.0 )
	void setDutyCycleVal ( GSSynthOsc oscillator , sbyte      dcVal );					// set the duty cycle as a signed byte

	void enableFreqTarget( GSSynthOsc oscillator , bool enable );						// enable or disable frequency targeting
	void setFreqTarget   ( GSSynthOsc oscillator , float frqHz , float relRate );		// set frequency target (Hz) with rel rate ( 0.0 to 1.0 )
	void setFreqTargetVal( GSSynthOsc oscillator , uint freqVal , uint rateVal );		// set frequency target as unsigned longs
	
	void enableFreqRamp  ( GSSynthOsc oscillator , bool enable );						// enable or disable frequency ramping
	void setFreqRamp     ( GSSynthOsc oscillator , float relRate );						// set an oscillator to sweep the entire frequency range at given rate
																					    // back and forth at a relative rate ( 0.0 - 1.0 )
	void setFreqRampVal  ( GSSynthOsc oscillator , uint relRateVal );					// set frequency ramp value as unsigned long

	void enableAmpTarget ( GSSynthOsc oscillator , bool enable );						// enable or disable amplitude targeting
	void setAmpTarget    ( GSSynthOsc oscillator , float amplitude , float relRate );	// set a target amplitude to move towards at a relative rate ( 0.0 to 1.0 )
																						// amplitude ( -1.0 to 1.0 ), 0.0 specified for rate disables
	void setAmpTargetVal ( GSSynthOsc oscillator , sbyte ampVal , uint relRateVal );	// set amplitude target as integers

	
	void enableAmpRamp   ( GSSynthOsc oscillator , bool enable );						// enable or disable amplitude ramping
	void setAmpRamp      ( GSSynthOsc oscillator , float timeSecs );					// set an oscillator to ramp down to zero in specified time
																						// time is actually a rate at assumed 100% amplitude
	void setAmpRampVal   ( GSSynthOsc oscIdx , uint relRateVal );						// set amplitude ramp by unsigned long
	

	void setEnvelope     ( GSSynthOsc  oscillator , 
					       GSAttackDur attackDur  , float attackAmp  ,					// set the ADSR envelope
					       GSDecRelDur decayDur   , float decayAmp   ,					// amplitudes vary from 0.0 to 1.0
					       GSDecRelDur releaseDur , float releaseAmp );					// durations are limited to selections in GinSingDefs.h

	void setEnvelopeVal  ( GSSynthOsc  oscillator ,										// set the ADSR envelope by unsigned byte
						   GSAttackDur attackDur  , ubyte attackAmpVal  ,
						   GSDecRelDur decayDur   , ubyte decayAmpVal   ,
						   GSDecRelDur releaseDur , ubyte releaseAmpVal );
	
	void enableBend    ( GSSynthOsc oscillator  , bool enable );						// enable or disable frequency bend to bank mixer
	
};


#endif // _GINSING_POLY_DEF

