
// ---------------------------------------------------------------------------------------------------------------------------------------------
// GinSingVoice - voice synthesis support the arduino GinSing shield
// ---------------------------------------------------------------------------------------------------------------------------------------------
// 

#include "GinSing.h"
#include "GinSingVoice.h"


// speech control register bits

#define SC_SpeedUpBit       0x01			// [0] speed up next allophone
#define SC_SpeedDownBit     0x02			// [1] slow down next allophone
#define SC_VolumeUpBit      0x04			// [2] increase volume on next allohone
#define SC_VolumeDownBit    0x08			// [3] decrease volume on next allphone
#define SC_PitchUpBit       0x10			// [4] increase pitch on next allophone
#define SC_PitchDownBit     0x20			// [5] decrease pitch on next allophone
#define SC_BendUpBit        0x40			// [6] bend pitch up on next allophone
#define SC_BendDownBit      0x80			// [7] bend pitch down on next allpohone

static ubyte _inflection = 0;				// inflection to be applied on the next allophone



// allophone duration table
// ---------------------------------------------------------------------------------------------------------------------------------------------

int alloDurs[] =
{
	/* _A   =  */  70 , /* _AA  =  */  70 , /* _AE  = */ 165 ,  /* _AIR = */ 200 ,  /* _AU  =  */  70 ,  /* _BE  =  */   45 , 
	/* _BO  =  */  45 , /* _EB  =  */  40 , /* _OB  = */  40 ,  /* _CH  = */  70 ,  /* _DE  =  */  45 ,  /* _DO  =  */   45 ,
	/* _ED  =  */  40 , /* _OD  =  */  40 , /* _E   = */  70 ,  /* _EE  = */  70 ,  /* _ER  =  */ 199 ,  /* _F   =  */   70 ,
	/* _GE  =  */  55 , /* _GO  =  */  55 , /* _HE  = */  70 ,  /* _HO  = */  70 ,  /* _I   =  */  70 ,  /* _IE  =  */  190 ,	
	/* _J   =  */  70 , /* _KE  =  */  55 , /* _KO  = */  55 ,	/* _EK  = */  45 ,  /* _OK  =  */  45 ,  /* _LE  =  */   70 ,
	/* _LO  =  */  70 , /* _M   =  */  70 ,	/* _NE  = */  70 ,	/* _NO  = */  70 ,  /* _NGE =  */  70 ,  /* _NGO =  */   70 ,
	/* _OE  =  */ 170 , /* _OI  =  */ 225 , /* _OO  = */  70 ,  /* _OU  = */ 200 ,  /* _OR  =  */ 185 ,  /* _PE  =  */   70 ,
	/* _PO  =  */  70 , /* _R   =  */  70 , /* _SE  = */  70 ,  /* _SO  = */  70 ,  /* _SH  =  */  50 ,  /* _T   =  */   45 ,
	/* _TH  =  */  40 , /* _THH =  */  70 , /* _TS  = */ 110 ,  /* _U   = */  70 ,  /* _UE  =  */  70 ,  /* _V   =  */   70 ,
	/* _W   =  */  20 , /* _Y   =  */  20 , /* _Z   = */  70 ,  /* _ZH  = */  70 ,  /* _PA0 =  */  12 ,  /* _PA1 =  */   48 ,
	/* _PA2 =  */  62 , /* _FD0 =  */   1 , /* _FD1 = */ 100 ,  /* _FD2 = */ 600 
};


// ---------------------------------------------------------------------------------------------------------------------------------------------
// begin - configure thye system for voice mode synthesis
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingVoice::begin ()
{	
	// reset the master settings
	
	gsPtr->getMaster()->reset ();
	
	// reset mixers
	
	gsPtr->sendCommand ( ResetToVoice_A );
	gsPtr->sendCommand ( ResetToVoice_B );

	
	// set default voice values
	
	gsPtr->sendCommand ( SetVoiceDefaults );
	
	
	// clear any old inflections
	
	_inflection = 0;
	
}

	
// ---------------------------------------------------------------------------------------------------------------------------------------------
// preview - preview voice mode
// ---------------------------------------------------------------------------------------------------------------------------------------------
	
void GinSingVoice::preview()
{
	// sqy hello
	
	GSAllophone  hello[] = {  _HE , _PITCHUP , _E , _LO , _BENDDN , _OE , _PA1 , _PA1 , _PA1 , 
	                          _W , _BENDDN , _ER , _BENDDN , _LE , _ED , _PA1 , _ENDPHRASE }; 
	speak ( hello );
	
	delay ( 2 * getMillis( hello ) );
	
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// speak - send a series of speech fragments (allophones) to the sound chip
// ---------------------------------------------------------------------------------------------------------------------------------------------

void  GinSingVoice::speak ( GSAllophone *allophoneList )
{	
	// write allophone bytes until we hit a zero
	
	while ( (*allophoneList) != _ENDPHRASE )
	{
		ubyte dataByte  = (*allophoneList);

		
		// the byte may be an inflection - if so build the inflection register byte
		
		if ( dataByte < _A )
		{
			switch ( dataByte )
			{
			case _SPEEDUP : _inflection |= SC_SpeedUpBit;    break;
			case _SPEEDDN : _inflection |= SC_SpeedDownBit;  break;
			case _VOLUP   : _inflection |= SC_VolumeUpBit;   break;
			case _VOLDN   : _inflection |= SC_VolumeDownBit; break;
			case _PITCHUP : _inflection |= SC_PitchUpBit;    break;
			case _PITCHDN : _inflection |= SC_PitchDownBit;  break;
			case _BENDUP  : _inflection |= SC_BendUpBit;     break;
			case _BENDDN  : _inflection |= SC_BendDownBit;   break;
			default       :                                  break;  
			};
		}
		
		// if its not an inflection write the allophone and set the speech control register as needed
		
		else
		{
			if ( _inflection )
			{
				gsPtr->sendCommand ( WriteOneByte , SpeechControl , _inflection ); _inflection = 0;
			}
			
			gsPtr->getSerial()->write ( (*allophoneList) );

		}
		
		allophoneList++;
	}
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// getMillis - compute the time length (ms) of a phrase
// ---------------------------------------------------------------------------------------------------------------------------------------------

int GinSingVoice::getMillis ( GSAllophone *allophoneList )
{
	int totalMS = 0;
	
	// go through the list and accumulate time from duration table
	// the table is indexed relative to the first allophone ( _A ) and inflections are skipped
	
	while ( (*allophoneList) != _ENDPHRASE )
	{
		ubyte dataByte = (*allophoneList);
		
		if ( dataByte >= (int)_A )
		{
			int durIdx = (int) dataByte - (int)_A;
			totalMS += alloDurs [ durIdx ];
		}
		allophoneList++;
	}
	
	return ( totalMS );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setBlendSpeed - set the relative blending speed bewteen phonemes
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingVoice::setBlendSpeed ( float relSpeed )
{
	ubyte blendByte = (ubyte) ( 255.0f * relSpeed );
	
	// set the voice transition speed register
	
	gsPtr->sendCommand ( WriteOneByte , TransitionSpeed , blendByte );
}

// ---------------------------------------------------------------------------------------------------------------------------------------------
// setBlendSpeed - set the relative blending speed bewteen phonemes
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingVoice::setDelay( float relDelay )
{
	ubyte delayByte = (ubyte) ( 255.0f * relDelay );
	
	// send the voice delay command
	
	gsPtr->sendCommand ( SetVoiceDelay , delayByte );
}


// ---------------------------------------------------------------------------------------------------------------------------------------------
// setNote - set the voice base pitch from  a musical note
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingVoice::setNote ( GSNote noteIdx )
{
	// send the note command
	
	gsPtr->sendCommand ( SetVoiceNote , noteIdx );
}
	
// ---------------------------------------------------------------------------------------------------------------------------------------------
// setFrequency - set the voice base pitch from a specified frequency (Hz)
// ---------------------------------------------------------------------------------------------------------------------------------------------

void GinSingVoice::setFrequency ( float freqHz )
{
	// convert target frequency to 2 byte value
	
	unsigned long freqInt = (unsigned long) ( freqHz / 0.0009313226 );
	ubyte freqLo   = ( freqInt & 0x00ff00 ) >> 8;
	ubyte freqHi   = ( freqInt & 0xff0000 ) >> 16;	
	
	
	// send the voice frequency command
	
	gsPtr->sendCommand ( SetVoiceFrequency , freqLo , freqHi );
}


					  
